#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from cinderclient import exceptions as cinder_exceptions
import copy
from oslo_log import log as logging
import six
from tempest.lib import decorators

from heat_tempest_plugin.common import exceptions
from heat_tempest_plugin.common import test
from heat_tempest_plugin.tests.scenario import scenario_base

LOG = logging.getLogger(__name__)


@test.requires_service_feature('volume', 'backup')
class VolumeBackupRestoreIntegrationTest(scenario_base.ScenarioTestsBase):
    """Class is responsible for testing of volume backup."""

    def setUp(self):
        super(VolumeBackupRestoreIntegrationTest, self).setUp()
        self.volume_description = 'A test volume description 123'
        self.volume_size = self.conf.volume_size

    def _cinder_verify(self, volume_id, expected_status='available'):
        self.assertIsNotNone(volume_id)
        volume = self.volume_client.volumes.get(volume_id)
        self.assertIsNotNone(volume)
        self.assertEqual(expected_status, volume.status)
        self.assertEqual(self.volume_size, volume.size)
        self.assertEqual(self.volume_description, volume.description)

    def _outputs_verify(self, stack, expected_status='available'):
        self.assertEqual(expected_status,
                         self._stack_output(stack, 'status'))
        self.assertEqual(six.text_type(self.volume_size),
                         self._stack_output(stack, 'size'))
        self.assertEqual(self.volume_description,
                         self._stack_output(stack, 'display_description'))

    def check_stack(self, stack_id, default_parameters):
        stack = self.client.stacks.get(stack_id)

        # Verify with cinder that the volume exists, with matching details
        volume_id = self._stack_output(stack, 'volume_id')
        self._cinder_verify(volume_id, expected_status='in-use')

        # Verify the stack outputs are as expected
        self._outputs_verify(stack, expected_status='in-use')

        # Delete the stack and ensure a backup is created for volume_id
        # but the volume itself is gone
        self._stack_delete(stack_id)
        self.assertRaises(cinder_exceptions.NotFound,
                          self.volume_client.volumes.get,
                          volume_id)

        backups = self.volume_client.backups.list()
        self.assertIsNotNone(backups)
        backups_filtered = [b for b in backups if b.volume_id == volume_id]
        self.assertEqual(1, len(backups_filtered))
        backup = backups_filtered[0]
        self.addCleanup(self.volume_client.backups.delete, backup.id)

        parameters = copy.deepcopy(default_parameters)
        parameters.update({'backup_id': backup.id})
        # Now, we create another stack where the volume is created from the
        # backup created by the previous stack
        try:
            stack_identifier2 = self.launch_stack(
                template_name='test_volumes_create_from_backup.yaml',
                parameters=parameters)
            stack2 = self.client.stacks.get(stack_identifier2)
        except exceptions.StackBuildErrorException:
            LOG.exception("Halting test due to bug: #1382300")
            return

        # Verify with cinder that the volume exists, with matching details
        volume_id2 = self._stack_output(stack2, 'volume_id')
        self._cinder_verify(volume_id2, expected_status='in-use')

        # Verify the stack outputs are as expected
        self._outputs_verify(stack2, expected_status='in-use')
        testfile_data = self._stack_output(stack2, 'testfile_data')
        self.assertEqual('{"instance1": "Volume Data:ateststring"}',
                         testfile_data)

        # Delete the stack and ensure the volume is gone
        self._stack_delete(stack_identifier2)
        self.assertRaises(cinder_exceptions.NotFound,
                          self.volume_client.volumes.get,
                          volume_id2)

    @decorators.idempotent_id('c3416735-87bf-4478-85c5-b3823819eb19')
    def test_cinder_volume_create_backup_restore(self):
        """Ensure the 'Snapshot' deletion policy works.

        This requires a more complex test, but it tests several aspects
        of the heat cinder resources:
           1. Create a volume, attach it to an instance, write some data to it
           2. Delete the stack, with 'Snapshot' specified, creates a backup
           3. Check the snapshot has created a volume backup
           4. Create a new stack, where the volume is created from the backup
           5. Verify the test data written in (1) is present in the new volume
        """
        default_parameters = {
            'key_name': self.keypair_name,
            'instance_type': self.conf.minimal_instance_type,
            'image_id': self.conf.minimal_image_ref,
            'volume_description': self.volume_description,
            'timeout': self.conf.build_timeout,
            'network': self.net['id']
        }
        if self.conf.vm_to_heat_api_insecure:
            default_parameters['wc_extra_args'] = '--insecure'

        parameters = copy.deepcopy(default_parameters)
        parameters.update({'volume_size': self.volume_size})
        # Launch stack
        stack_id = self.launch_stack(
            template_name='test_volumes_delete_snapshot.yaml',
            parameters=parameters
        )

        # Check stack
        self.check_stack(stack_id, default_parameters)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import yaml

from heatclient.common import template_utils
from oslo_utils import reflection

from heat_tempest_plugin.common import test


class ScenarioTestsBase(test.HeatIntegrationTest):
    """This class defines common parameters for scenario tests."""

    def setUp(self):
        super(ScenarioTestsBase, self).setUp()
        self.check_skip()
        self.sub_dir = 'templates'
        self.assign_keypair()

        if not self.conf.fixed_network_name:
            raise self.skipException("No default network configured to test")
        self.net = self._get_network()

        if not self.conf.minimal_image_ref:
            raise self.skipException("No minimal image configured to test")
        if not self.conf.minimal_instance_type:
            raise self.skipException("No minimal flavor configured to test")

    def _load_template(self, base_file, file_name, sub_dir=None, files=None):
        sub_dir = sub_dir or ''
        filepath = os.path.join(os.path.dirname(os.path.realpath(base_file)),
                                sub_dir, file_name)
        _files, template = template_utils.get_template_contents(filepath,
                                                                files=files)
        return yaml.safe_dump(template)

    def launch_stack(self, template_name, expected_status='CREATE_COMPLETE',
                     parameters=None, **kwargs):
        files = kwargs.get('files', {})
        template = self._load_template(__file__, template_name, self.sub_dir,
                                       files)

        stack_id = self.stack_create(
            stack_name=kwargs.get('stack_name'),
            template=template,
            files=files,
            parameters=parameters,
            environment=kwargs.get('environment'),
            expected_status=expected_status
        )

        return stack_id

    def check_skip(self):
        test_cls_name = reflection.get_class_name(self, fully_qualified=False)
        test_method_name = '.'.join([test_cls_name, self._testMethodName])
        test_skipped = (self.conf.skip_scenario_test_list and (
            test_cls_name in self.conf.skip_scenario_test_list or
            test_method_name in self.conf.skip_scenario_test_list))
        if self.conf.skip_scenario_tests or test_skipped:
            self.skipTest('Test disabled in conf, skipping')



# Copyright 2019 Ericsson Software Technology
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import yaml

from oslo_log import log
from tempest.lib import decorators

from heat_tempest_plugin.common import test
from heat_tempest_plugin.tests.functional import functional_base


LOG = log.getLogger(__name__)

test_template = '''
heat_template_version: pike
description: Test template for OS::Neutron::ExtraRouteSet
resources:
  net0:
    type: OS::Neutron::Net
  subnet0:
    type: OS::Neutron::Subnet
    properties:
      network: { get_resource: net0 }
      cidr: 10.0.0.0/24
  router0:
    type: OS::Neutron::Router
  routerinterface0:
    type: OS::Neutron::RouterInterface
    properties:
      router: { get_resource: router0 }
      subnet: { get_resource: subnet0 }
  extrarouteset0:
    type: OS::Neutron::ExtraRouteSet
    properties:
      router: { get_resource: router0 }
      routes:
        - destination: 10.0.10.0/24
          nexthop: 10.0.0.10
        - destination: 10.0.11.0/24
          nexthop: 10.0.0.11
'''


def _routes_to_set(routes):
    '''Convert a list of extra routes to an unordered data type.'''
    return set(frozenset(r.items()) for r in routes)


@test.requires_resource_type('OS::Neutron::ExtraRouteSet')
class ExtraRouteSetTest(functional_base.FunctionalTestsBase):

    def _create(self, template_routes):
        parsed_template = yaml.safe_load(test_template)
        parsed_template['resources'][
            'extrarouteset0']['properties']['routes'] = template_routes
        create_template = yaml.safe_dump(parsed_template)

        stack_id = self.stack_create(template=create_template)

        neutron_router_id = self.get_physical_resource_id(
            stack_id, 'router0')
        neutron_router = self.network_client.show_router(
            neutron_router_id)['router']
        neutron_routes = neutron_router['routes']

        self.assertEqual(
            _routes_to_set(template_routes),
            _routes_to_set(neutron_routes))

    @decorators.idempotent_id('95b92c1e-d082-11e9-9e5d-9bdb7311b69b')
    def test_create_no(self):
        self._create(template_routes=[])

    @decorators.idempotent_id('abda6884-d0b2-11e9-9819-4f4e24c86c92')
    def test_create_one(self):
        self._create(template_routes=[
            {'destination': '10.0.10.0/24', 'nexthop': '10.0.0.10'},
        ])

    @decorators.idempotent_id('b3a9b6be-d0b2-11e9-a504-7b3ba6df5e7a')
    def test_create_many(self):
        self._create(template_routes=[
            {'destination': '10.0.10.0/24', 'nexthop': '10.0.0.10'},
            {'destination': '10.0.11.0/24', 'nexthop': '10.0.0.11'},
        ])

    def _update(self, template_routes):
        stack_id = self.stack_create(template=test_template)

        parsed_template = yaml.safe_load(test_template)
        parsed_template['resources'][
            'extrarouteset0']['properties']['routes'] = template_routes
        updated_template = yaml.safe_dump(parsed_template)
        self.update_stack(stack_id, updated_template)

        neutron_router_id = self.get_physical_resource_id(
            stack_id, 'router0')
        neutron_router = self.network_client.show_router(
            neutron_router_id)['router']
        neutron_routes = neutron_router['routes']

        self.assertEqual(
            _routes_to_set(template_routes),
            _routes_to_set(neutron_routes))

    @decorators.idempotent_id('6dcf2110-d0b7-11e9-b26b-9f2e5e98d09d')
    def test_update_no(self):
        self._update(template_routes=[])

    @decorators.idempotent_id('6e3cea1a-d0b7-11e9-aebc-f7922b833ea5')
    def test_update_one(self):
        self._update(template_routes=[
            {'destination': '10.0.10.0/24', 'nexthop': '10.0.0.10'},
        ])

    @decorators.idempotent_id('6e970126-d0b7-11e9-8ec8-afde1e726d0f')
    def test_update_many(self):
        self._update(template_routes=[
            {'destination': '10.0.10.0/24', 'nexthop': '10.0.0.10'},
            {'destination': '10.0.11.0/24', 'nexthop': '10.0.0.11'},
        ])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


class OSWaitCondition(functional_base.FunctionalTestsBase):

    template = '''
heat_template_version: 2013-05-23
parameters:
  flavor:
    type: string
  image:
    type: string
  network:
    type: string
  timeout:
    type: number
    default: 60
  wc_extra_args:
    type: string
    default: ""
resources:
  instance1:
    type: OS::Nova::Server
    properties:
      flavor: {get_param: flavor}
      image: {get_param: image}
      networks:
      - network: {get_param: network}
      user_data_format: RAW
      user_data:
        str_replace:
          template: '#!/bin/sh

            while true; do wc_notify --data-binary ''{"status":
            "SUCCESS"}''; if [ $? -eq 0 ]; then break; else sleep 10; fi; done

            # signals with reason

            while true; do wc_notify --data-binary ''{"status":
            "SUCCESS", "reason":
            "signal2"}''; if [ $? -eq 0 ]; then break; else sleep 10; fi; done

            # signals with data

            while true; do wc_notify --data-binary ''{"status":
            "SUCCESS", "reason": "signal3", "data":
            "data3"}''; if [ $? -eq 0 ]; then break; else sleep 10; fi; done

            while true; do wc_notify --data-binary ''{"status":
            "SUCCESS", "reason": "signal4", "data":
            "data4"}''; if [ $? -eq 0 ]; then break; else sleep 10; fi; done

            # check signals with the same ID

            while true; do wc_notify --data-binary ''{"status":
            "SUCCESS", "id":
            "test5"}''; if [ $? -eq 0 ]; then break; else sleep 10; fi; done

            while true; do wc_notify --data-binary ''{"status":
            "SUCCESS", "id":
            "test5"}''; if [ $? -eq 0 ]; then break; else sleep 10; fi; done

            _signal(){ while true; do wc_notify --data-binary ''{"status":
            "SUCCESS"}''; if [ $? -eq 0 ]; then break; fi; done }

            # loop for 20 signals without reasons and data

            for i in `seq 1 20`; do _signal & done

            wait
            '
          params:
            wc_notify:
              list_join:
                - " "
                - [ get_attr: [ wait_handle, curl_cli],
                    get_param: wc_extra_args ]

  wait_condition:
    type: OS::Heat::WaitCondition
    depends_on: instance1
    properties:
      count: 25
      handle: {get_resource: wait_handle}
      timeout: {get_param: timeout}

  wait_handle:
    type: OS::Heat::WaitConditionHandle

outputs:
  curl_cli:
    value:
      get_attr: [wait_handle, curl_cli]
  wc_data:
    value:
      get_attr: [wait_condition, data]
'''

    def setUp(self):
        super(OSWaitCondition, self).setUp()
        if not self.conf.minimal_image_ref:
            raise self.skipException("No minimal image configured to test")
        if not self.conf.minimal_instance_type:
            raise self.skipException("No minimal flavor configured to test")

    @decorators.idempotent_id('cc54ca6e-b91d-4ddd-80cc-24a886dfaaa0')
    def test_create_stack_with_multi_signal_waitcondition(self):
        params = {'flavor': self.conf.minimal_instance_type,
                  'image': self.conf.minimal_image_ref,
                  'network': self.conf.fixed_network_name,
                  'timeout': 180}
        if self.conf.vm_to_heat_api_insecure:
            params['wc_extra_args'] = '--insecure'
        self.stack_create(template=self.template, parameters=params)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


class StackUnicodeTemplateTest(functional_base.FunctionalTestsBase):

    random_template = u'''
heat_template_version: 2014-10-16
description: \u8fd9\u662f\u4e00\u4e2a\u63cf\u8ff0
parameters:
  \u53c2\u6570:
    type: number
    default: 10
    label: \u6807\u7b7e
    description: \u8fd9\u662f\u4e00\u4e2a\u63cf\u8ff0
resources:
  \u8d44\u6e90:
    type: OS::Heat::RandomString
    properties:
      length: {get_param: \u53c2\u6570}
outputs:
  \u8f93\u51fa:
    description: \u8fd9\u662f\u4e00\u4e2a\u63cf\u8ff0
    value: {get_attr: [\u8d44\u6e90, value]}
'''

    def _assert_results(self, result):
        self.assertTrue(result['disable_rollback'])
        self.assertIsNone(result['parent'])
        self.assertEqual(u'\u8fd9\u662f\u4e00\u4e2a\u63cf\u8ff0',
                         result['template_description'])
        self.assertEqual(u'10', result['parameters'][u'\u53c2\u6570'])

    def _assert_preview_results(self, result):
        self._assert_results(result)
        res = result['resources'][0]
        self.assertEqual('/resources/%s' % res['resource_name'],
                         res['resource_identity']['path'])

    def _assert_create_results(self, result):
        self._assert_results(result)
        output = result['outputs'][0]
        self.assertEqual(u'\u8fd9\u662f\u4e00\u4e2a\u63cf\u8ff0',
                         output['description'])
        self.assertEqual(u'\u8f93\u51fa', output['output_key'])
        self.assertIsNotNone(output['output_value'])

    def _assert_resource_results(self, result):
        self.assertEqual(u'\u8d44\u6e90', result['resource_name'])
        self.assertEqual('OS::Heat::RandomString',
                         result['resource_type'])

    @decorators.idempotent_id('871cc5dd-7550-494d-8c6d-9d549a8c0305')
    def test_template_validate_basic(self):
        ret = self.client.stacks.validate(template=self.random_template)
        expected = {
            'Description': u'\u8fd9\u662f\u4e00\u4e2a\u63cf\u8ff0',
            'Parameters': {
                u'\u53c2\u6570': {
                    'Default': 10,
                    'Description': u'\u8fd9\u662f\u4e00\u4e2a\u63cf\u8ff0',
                    'Label': u'\u6807\u7b7e',
                    'NoEcho': 'false',
                    'Type': 'Number'}
            },
            'Environment': {
                'event_sinks': [],
                'parameter_defaults': {},
                'parameters': {},
                'resource_registry': {u'resources': {}}
            }
        }
        self.assertEqual(expected, ret)

    @decorators.idempotent_id('e84c76b8-a5e5-45e7-9c4b-4c160d1c385f')
    def test_template_validate_override_default(self):
        env = {'parameters': {u'\u53c2\u6570': 5}}
        ret = self.client.stacks.validate(template=self.random_template,
                                          environment=env)
        expected = {
            'Description': u'\u8fd9\u662f\u4e00\u4e2a\u63cf\u8ff0',
            'Parameters': {
                u'\u53c2\u6570': {
                    'Default': 10,
                    'Value': 5,
                    'Description': u'\u8fd9\u662f\u4e00\u4e2a\u63cf\u8ff0',
                    'Label': u'\u6807\u7b7e',
                    'NoEcho': 'false',
                    'Type': 'Number'}
            },
            'Environment': {
                'event_sinks': [],
                'parameter_defaults': {},
                'parameters': {u'\u53c2\u6570': 5},
                'resource_registry': {u'resources': {}}
            }
        }
        self.assertEqual(expected, ret)

    @decorators.idempotent_id('66338945-a3ae-4e3a-aa17-ab802ceb00b1')
    def test_stack_preview(self):
        result = self.client.stacks.preview(
            template=self.random_template,
            stack_name=self._stack_rand_name(),
            disable_rollback=True).to_dict()
        self._assert_preview_results(result)

    @decorators.idempotent_id('d2c4a10c-3cb4-4efd-889d-695a0acbd04f')
    def test_create_stack(self):
        stack_identifier = self.stack_create(template=self.random_template)
        stack = self.client.stacks.get(stack_identifier)
        self._assert_create_results(stack.to_dict())
        rl = self.client.resources.list(stack_identifier)
        self.assertEqual(1, len(rl))
        self._assert_resource_results(rl[0].to_dict())



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heatclient import exc
import six
from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


class StackPreviewTest(functional_base.FunctionalTestsBase):
    template = '''
heat_template_version: 2015-04-30
parameters:
  incomming:
    type: string
resources:
  one:
    type: OS::Heat::TestResource
    properties:
      value: fred
  two:
    type: OS::Heat::TestResource
    properties:
      value: {get_param: incomming}
    depends_on: one
outputs:
  main_out:
    value: {get_attr: [two, output]}
    '''
    env = '''
parameters:
  incomming: abc
    '''

    def setUp(self):
        super(StackPreviewTest, self).setUp()
        self.client = self.orchestration_client
        self.project_id = self.identity_client.project_id

    def _assert_resource(self, res, stack_name):
        self.assertEqual(stack_name, res['stack_name'])
        self.assertEqual('INIT', res['resource_action'])
        self.assertEqual('COMPLETE', res['resource_status'])
        for field in ('resource_status_reason', 'physical_resource_id',
                      'description'):
            self.assertIn(field, res)
            self.assertEqual('', res[field])
        # 'creation_time' and 'updated_time' are None when preview
        for field in ('creation_time', 'updated_time'):
            self.assertIn(field, res)
            self.assertIsNone(res[field])
        self.assertIn('output', res['attributes'])

        # resource_identity
        self.assertEqual(stack_name,
                         res['resource_identity']['stack_name'])
        self.assertEqual('None', res['resource_identity']['stack_id'])
        self.assertEqual(self.project_id,
                         res['resource_identity']['tenant'])
        self.assertEqual('/resources/%s' % res['resource_name'],
                         res['resource_identity']['path'])
        # stack_identity
        self.assertEqual(stack_name,
                         res['stack_identity']['stack_name'])
        self.assertEqual('None', res['stack_identity']['stack_id'])
        self.assertEqual(self.project_id,
                         res['stack_identity']['tenant'])
        self.assertEqual('', res['stack_identity']['path'])

    def _assert_results(self, result, stack_name):
        # global stuff.
        self.assertEqual(stack_name, result['stack_name'])
        self.assertTrue(result['disable_rollback'])
        self.assertEqual('None', result['id'])
        self.assertIsNone(result['parent'])
        self.assertEqual('No description', result['template_description'])

        # parameters
        self.assertEqual('None', result['parameters']['OS::stack_id'])
        self.assertEqual(stack_name, result['parameters']['OS::stack_name'])
        self.assertEqual('abc', result['parameters']['incomming'])

    @decorators.idempotent_id('4c2c062d-4723-42c0-a113-3307dd4ceb12')
    def test_basic_pass(self):
        stack_name = self._stack_rand_name()
        result = self.client.stacks.preview(
            template=self.template,
            stack_name=stack_name,
            disable_rollback=True,
            environment=self.env).to_dict()

        self._assert_results(result, stack_name)
        for res in result['resources']:
            self._assert_resource(res, stack_name)
            self.assertEqual('OS::Heat::TestResource',
                             res['resource_type'])

            # common properties
            self.assertFalse(res['properties']['fail'])
            self.assertEqual(0, res['properties']['wait_secs'])
            self.assertFalse(res['properties']['update_replace'])

            if res['resource_name'] == 'one':
                self.assertEqual('fred', res['properties']['value'])
                self.assertEqual(['two'], res['required_by'])
            if res['resource_name'] == 'two':
                self.assertEqual('abc', res['properties']['value'])
                self.assertEqual([], res['required_by'])

    @decorators.idempotent_id('54e69a54-762d-479e-a4f4-1fbf21af01c6')
    def test_basic_fail(self):
        stack_name = self._stack_rand_name()

        # break the template so it fails validation.
        wont_work = self.template.replace('get_param: incomming',
                                          'get_param: missing')
        excp = self.assertRaises(exc.HTTPBadRequest,
                                 self.client.stacks.preview,
                                 template=wont_work,
                                 stack_name=stack_name,
                                 disable_rollback=True,
                                 environment=self.env)

        self.assertIn('Property error: : resources.two.properties.value: '
                      ': The Parameter (missing) was not provided.',
                      six.text_type(excp))

    @decorators.idempotent_id('0449113c-ff90-4f2b-8825-27ea35c1983f')
    def test_nested_pass(self):
        """Nested stacks need to recurse down the stacks."""
        main_template = '''
heat_template_version: 2015-04-30
parameters:
  incomming:
    type: string
resources:
  main:
    type: nested.yaml
    properties:
      value: {get_param: incomming}
outputs:
  main_out:
    value: {get_attr: [main, output]}
    '''
        nested_template = '''
heat_template_version: 2015-04-30
parameters:
  value:
    type: string
resources:
  nested:
    type: OS::Heat::TestResource
    properties:
      value: {get_param: value}
outputs:
  output:
    value: {get_attr: [nested, output]}
'''
        stack_name = self._stack_rand_name()
        result = self.client.stacks.preview(
            disable_rollback=True,
            stack_name=stack_name,
            template=main_template,
            files={'nested.yaml': nested_template},
            environment=self.env).to_dict()

        self._assert_results(result, stack_name)

        # nested resources return a list of their resources.
        res = result['resources'][0][0]
        nested_stack_name = '%s-%s' % (stack_name,
                                       res['parent_resource'])

        self._assert_resource(res, nested_stack_name)
        self.assertEqual('OS::Heat::TestResource',
                         res['resource_type'])

        self.assertFalse(res['properties']['fail'])
        self.assertEqual(0, res['properties']['wait_secs'])
        self.assertFalse(res['properties']['update_replace'])

        self.assertEqual('abc', res['properties']['value'])
        self.assertEqual([], res['required_by'])

    @decorators.idempotent_id('6ca8ddfc-106f-4ecc-83f7-fca31d0c85ca')
    def test_res_group_with_nested_template(self):
        main_template = '''
heat_template_version: 2015-04-30
resources:
  fixed_network:
    type: "OS::Neutron::Net"
  rg:
    type: "OS::Heat::ResourceGroup"
    properties:
      count: 1
      resource_def:
        type: nested.yaml
        properties:
          fixed_network_id: {get_resource: fixed_network}
    '''
        nested_template = '''
heat_template_version: 2015-04-30

parameters:
  fixed_network_id:
    type: string
resources:
  port:
    type: "OS::Neutron::Port"
    properties:
      network_id:
          get_param: fixed_network_id

'''
        stack_name = self._stack_rand_name()
        result = self.client.stacks.preview(
            disable_rollback=True,
            stack_name=stack_name,
            template=main_template,
            files={'nested.yaml': nested_template}).to_dict()

        resource_names = []

        def get_resource_names(resources):
            for item in resources:
                if isinstance(item, dict):
                    resource_names.append(item['resource_name'])
                else:
                    get_resource_names(item)
        get_resource_names(result['resources'])
        # ensure that fixed network and port here
        self.assertIn('fixed_network', resource_names)
        self.assertIn('port', resource_names)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


class StackOutputsTest(functional_base.FunctionalTestsBase):

    template = '''
heat_template_version: 2015-10-15
resources:
  test_resource_a:
    type: OS::Heat::TestResource
    properties:
      value: 'a'
  test_resource_b:
    type: OS::Heat::TestResource
    properties:
      value: 'b'
outputs:
  resource_output_a:
    description: 'Output of resource a'
    value: { get_attr: [test_resource_a, output] }
  resource_output_b:
    description: 'Output of resource b'
    value: { get_attr: [test_resource_b, output] }
'''

    @decorators.idempotent_id('a886dd67-4506-4a37-82ae-43f0a7d83f35')
    def test_outputs(self):
        stack_identifier = self.stack_create(
            template=self.template
        )
        expected_list = [{u'output_key': u'resource_output_a',
                          u'description': u'Output of resource a'},
                         {u'output_key': u'resource_output_b',
                          u'description': u'Output of resource b'}]

        actual_list = self.client.stacks.output_list(
            stack_identifier)['outputs']
        sorted_actual_list = sorted(actual_list, key=lambda x: x['output_key'])
        self.assertEqual(expected_list, sorted_actual_list)

        expected_output_a = {
            u'output_value': u'a', u'output_key': u'resource_output_a',
            u'description': u'Output of resource a'}
        expected_output_b = {
            u'output_value': u'b', u'output_key': u'resource_output_b',
            u'description': u'Output of resource b'}
        actual_output_a = self.client.stacks.output_show(
            stack_identifier, 'resource_output_a')['output']
        actual_output_b = self.client.stacks.output_show(
            stack_identifier, 'resource_output_b')['output']
        self.assertEqual(expected_output_a, actual_output_a)
        self.assertEqual(expected_output_b, actual_output_b)

    before_template = '''
heat_template_version: 2015-10-15
resources:
  test_resource_a:
    type: OS::Heat::TestResource
    properties:
      value: 'foo'
outputs:
'''

    after_template = '''
heat_template_version: 2015-10-15
resources:
  test_resource_a:
    type: OS::Heat::TestResource
    properties:
      value: 'foo'
  test_resource_b:
    type: OS::Heat::TestResource
    properties:
      value: {get_attr: [test_resource_a, output]}
outputs:
  output_value:
    description: 'Output of resource b'
    value: {get_attr: [test_resource_b, output]}
'''

    @decorators.idempotent_id('aea0e495-4c77-4033-8c43-3351e9cb7b48')
    def test_outputs_update_new_resource(self):
        stack_identifier = self.stack_create(template=self.before_template)
        self.update_stack(stack_identifier, template=self.after_template)

        expected_output_value = {
            u'output_value': u'foo', u'output_key': u'output_value',
            u'description': u'Output of resource b'}
        actual_output_value = self.client.stacks.output_show(
            stack_identifier, 'output_value')['output']
        self.assertEqual(expected_output_value, actual_output_value)

    nested_template = '''
heat_template_version: 2015-10-15
resources:
  parent:
    type: 1.yaml
outputs:
  resource_output_a:
    value: { get_attr: [parent, resource_output_a] }
    description: 'parent a'
  resource_output_b:
    value: { get_attr: [parent, resource_output_b] }
    description: 'parent b'
    '''
    error_template = '''
heat_template_version: 2015-10-15
resources:
  test_resource_a:
    type: OS::Heat::TestResource
    properties:
      value: 'a'
  test_resource_b:
    type: OS::Heat::TestResource
    properties:
      value: 'b'
outputs:
  resource_output_a:
    description: 'Output of resource a'
    value: { get_attr: [test_resource_a, output] }
  resource_output_b:
    description: 'Output of resource b'
    value: { get_param: foo }
'''

    @decorators.idempotent_id('993a403c-c6e2-475d-a65d-a82b8c9e0c22')
    def test_output_error_nested(self):
        stack_identifier = self.stack_create(
            template=self.nested_template,
            files={'1.yaml': self.error_template}
        )
        self.update_stack(stack_identifier, template=self.nested_template,
                          files={'1.yaml': self.error_template})
        expected_list = [{u'output_key': u'resource_output_a',
                          u'output_value': u'a',
                          u'description': u'parent a'},
                         {u'output_key': u'resource_output_b',
                          u'output_value': None,
                          u'output_error': u'Error in parent output '
                                           u'resource_output_b: The Parameter'
                                           u' (foo) was not provided.',
                          u'description': u'parent b'}]

        actual_list = self.client.stacks.get(stack_identifier).outputs
        sorted_actual_list = sorted(actual_list, key=lambda x: x['output_key'])
        self.assertEqual(expected_list, sorted_actual_list)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os
import requests
import subprocess
import sys
import tempfile
import time
import yaml

from oslo_utils import timeutils
from tempest.lib import decorators

from heat_tempest_plugin.common import exceptions
from heat_tempest_plugin.common import test
from heat_tempest_plugin.tests.functional import functional_base


class ParallelDeploymentsTest(functional_base.FunctionalTestsBase):
    server_template = '''
heat_template_version: "2013-05-23"
parameters:
  flavor:
    type: string
  image:
    type: string
  network:
    type: string
resources:
  server:
    type: OS::Nova::Server
    properties:
      image: {get_param: image}
      flavor: {get_param: flavor}
      user_data_format: SOFTWARE_CONFIG
      networks: [{network: {get_param: network}}]
outputs:
  server:
    value: {get_resource: server}
'''

    config_template = '''
heat_template_version: "2013-05-23"
parameters:
  server:
    type: string
resources:
  config:
    type: OS::Heat::SoftwareConfig
    properties:
'''

    deployment_snippet = '''
type: OS::Heat::SoftwareDeployments
properties:
  config: {get_resource: config}
  servers: {'0': {get_param: server}}
'''

    enable_cleanup = True

    @decorators.idempotent_id('8ee231ff-f80a-4a17-a860-5cda87e18ad0')
    def test_deployments_metadata(self):
        parms = {'flavor': self.conf.minimal_instance_type,
                 'network': self.conf.fixed_network_name,
                 'image': self.conf.minimal_image_ref}
        stack_identifier = self.stack_create(
            parameters=parms,
            template=self.server_template,
            enable_cleanup=self.enable_cleanup)
        server_stack = self.client.stacks.get(stack_identifier)
        server = server_stack.outputs[0]['output_value']

        config_stacks = []
        # add up to 3 stacks each with up to 3 deployments
        deploy_count = 0
        deploy_count = self.deploy_many_configs(
            stack_identifier,
            server,
            config_stacks,
            2,
            5,
            deploy_count)
        self.deploy_many_configs(
            stack_identifier,
            server,
            config_stacks,
            3,
            3,
            deploy_count)

        self.signal_deployments(stack_identifier)
        for config_stack in config_stacks:
            self._wait_for_stack_status(config_stack, 'CREATE_COMPLETE')

    @decorators.idempotent_id('bd539232-b999-4bec-b47d-ff4822fc8b82')
    def test_deployments_timeout_failed(self):
        parms = {'flavor': self.conf.minimal_instance_type,
                 'network': self.conf.fixed_network_name,
                 'image': self.conf.minimal_image_ref}
        stack_identifier = self.stack_create(
            parameters=parms,
            template=self.server_template,
            enable_cleanup=self.enable_cleanup)
        server_stack = self.client.stacks.get(stack_identifier)
        server = server_stack.outputs[0]['output_value']
        config_stack = self.deploy_config(server, 3, 1)
        self._wait_for_stack_status(config_stack, 'CREATE_FAILED')
        kwargs = {'server_id': server}

        def check_deployment_status():
            sd_list = self.client.software_deployments.list(**kwargs)
            for sd in sd_list:
                if sd.status != 'FAILED':
                    return False
            return True

        self.assertTrue(test.call_until_true(
            20, 0, check_deployment_status))

    def deploy_many_configs(self, stack, server, config_stacks,
                            stack_count, deploys_per_stack,
                            deploy_count_start):
        for a in range(stack_count):
            config_stacks.append(
                self.deploy_config(server, deploys_per_stack))

        new_count = deploy_count_start + stack_count * deploys_per_stack
        self.wait_for_deploy_metadata_set(stack, new_count)
        return new_count

    def deploy_config(self, server, deploy_count, timeout=None):
        parms = {'server': server}
        template = yaml.safe_load(self.config_template)
        resources = template['resources']
        resources['config']['properties'] = {'config': 'x' * 10000}
        for a in range(deploy_count):
            resources['dep_%s' % a] = yaml.safe_load(self.deployment_snippet)
        return self.stack_create(
            parameters=parms,
            template=template,
            enable_cleanup=self.enable_cleanup,
            expected_status=None,
            timeout=timeout)

    def wait_for_deploy_metadata_set(self, stack, deploy_count):
        build_timeout = self.conf.build_timeout
        build_interval = self.conf.build_interval

        start = timeutils.utcnow()
        while timeutils.delta_seconds(start,
                                      timeutils.utcnow()) < build_timeout:
            server_metadata = self.client.resources.metadata(
                stack, 'server')
            if len(server_metadata['deployments']) == deploy_count:
                return
            time.sleep(build_interval)

        message = ('Deployment resources failed to be created within '
                   'the required time (%s s).' %
                   (build_timeout))
        raise exceptions.TimeoutException(message)

    def signal_deployments(self, stack_identifier):
        server_metadata = self.client.resources.metadata(
            stack_identifier, 'server')
        for dep in server_metadata['deployments']:
            iv = dict((i['name'], i['value']) for i in dep['inputs'])
            sigurl = iv.get('deploy_signal_id')
            requests.post(sigurl, data='{}',
                          headers={'content-type': 'application/json'},
                          verify=self.verify_cert)


@test.requires_service_type('messaging')
class ZaqarSignalTransportTest(functional_base.FunctionalTestsBase):
    server_template = '''
heat_template_version: "2013-05-23"

parameters:
  flavor:
    type: string
  image:
    type: string
  network:
    type: string

resources:
  server:
    type: OS::Nova::Server
    properties:
      image: {get_param: image}
      flavor: {get_param: flavor}
      user_data_format: SOFTWARE_CONFIG
      software_config_transport: ZAQAR_MESSAGE
      networks: [{network: {get_param: network}}]
  config:
    type: OS::Heat::SoftwareConfig
    properties:
      config: echo 'foo'
  deployment:
    type: OS::Heat::SoftwareDeployment
    properties:
      config: {get_resource: config}
      server: {get_resource: server}
      signal_transport: ZAQAR_SIGNAL

outputs:
  data:
    value: {get_attr: [deployment, deploy_stdout]}
'''

    conf_template = '''
[zaqar]
user_id = %(user_id)s
password = %(password)s
project_id = %(project_id)s
auth_url = %(auth_url)s
queue_id = %(queue_id)s
    '''

    @decorators.idempotent_id('3af97ced-bead-4629-b78a-97762719e990')
    def test_signal_queues(self):
        parms = {'flavor': self.conf.minimal_instance_type,
                 'network': self.conf.fixed_network_name,
                 'image': self.conf.minimal_image_ref}
        stack_identifier = self.stack_create(
            parameters=parms,
            template=self.server_template,
            expected_status=None)
        metadata = self.wait_for_deploy_metadata_set(stack_identifier)
        config = metadata['os-collect-config']['zaqar']
        conf_content = self.conf_template % config
        fd, temp_path = tempfile.mkstemp()
        os.write(fd, conf_content.encode('utf-8'))
        os.close(fd)
        cmd = ['os-collect-config', '--one-time',
               '--config-file=%s' % temp_path, 'zaqar']
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
        stdout_value = proc.communicate()[0]
        data = json.loads(stdout_value.decode('utf-8'))
        self.assertEqual(config, data['zaqar']['os-collect-config']['zaqar'])
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
        stdout_value = proc.communicate()[0]
        data = json.loads(stdout_value.decode('utf-8'))

        fd, temp_path = tempfile.mkstemp()
        os.write(fd,
                 json.dumps(data['zaqar']['deployments'][0]).encode('utf-8'))
        os.close(fd)
        cmd = [sys.executable, self.conf.heat_config_notify_script, temp_path]
        proc = subprocess.Popen(cmd,
                                stderr=subprocess.PIPE,
                                stdin=subprocess.PIPE)
        proc.communicate(
            json.dumps({'deploy_stdout': 'here!'}).encode('utf-8'))
        self._wait_for_stack_status(stack_identifier, 'CREATE_COMPLETE')
        stack = self.client.stacks.get(stack_identifier)
        self.assertEqual('here!', stack.outputs[0]['output_value'])

    def wait_for_deploy_metadata_set(self, stack):
        build_timeout = self.conf.build_timeout
        build_interval = self.conf.build_interval

        start = timeutils.utcnow()
        while timeutils.delta_seconds(start,
                                      timeutils.utcnow()) < build_timeout:
            server_metadata = self.client.resources.metadata(
                stack, 'server')
            if server_metadata.get('deployments'):
                return server_metadata
            time.sleep(build_interval)

        message = ('Deployment resources failed to be created within '
                   'the required time (%s s).' %
                   (build_timeout))
        raise exceptions.TimeoutException(message)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""A test module to exercise the Heat API with gabbi.  """

import os
import sys
import unittest

from gabbi import driver
import keystoneauth1
from oslo_log import log as logging
import six
from tempest import config

from heat_tempest_plugin.common import test
from heat_tempest_plugin.services import clients
from heat_tempest_plugin.tests.api import fixtures

LOG = logging.getLogger(__name__)
TESTS_DIR = 'gabbits'


def load_tests(loader, tests, pattern):
    """Provide a TestSuite to the discovery process."""
    test_dir = os.path.join(os.path.dirname(__file__), TESTS_DIR)

    endpoint = None
    conf = config.CONF.heat_plugin
    if conf.auth_url:
        try:
            manager = clients.ClientManager(conf)
            endpoint = manager.identity_client.get_endpoint_url(
                'orchestration', region=conf.region,
                endpoint_type=conf.endpoint_type)
            os.environ['PREFIX'] = test.rand_name('api')

        # Catch the authentication exceptions that can happen if one of the
        # following conditions occur:
        #   1. conf.auth_url IP/port is incorrect or keystone not available
        #      (ConnectFailure)
        #   2. conf.auth_url is malformed (BadRequest, UnknownConnectionError,
        #      EndpointNotFound, NotFound, or DiscoveryFailure)
        #   3. conf.username/password is incorrect (Unauthorized)
        #   4. conf.project_name is missing/incorrect (EmptyCatalog)
        # These exceptions should not prevent a test list from being returned,
        # so just issue a warning log and move forward with test listing.
        except (keystoneauth1.exceptions.http.BadRequest,
                keystoneauth1.exceptions.http.Unauthorized,
                keystoneauth1.exceptions.http.NotFound,
                keystoneauth1.exceptions.catalog.EmptyCatalog,
                keystoneauth1.exceptions.catalog.EndpointNotFound,
                keystoneauth1.exceptions.discovery.DiscoveryFailure,
                keystoneauth1.exceptions.connection.UnknownConnectionError,
                keystoneauth1.exceptions.connection.ConnectFailure):
            LOG.warn("Keystone auth exception: %s: %s" % (sys.exc_info()[0],
                                                          sys.exc_info()[1]))
            # Clear the auth_url, as there is no point in tempest trying
            # to authenticate later with mis-configured or unreachable endpoint
            conf.auth_url = None

        except Exception:
            LOG.error("Fatal exception: %s: %s" % (sys.exc_info()[0],
                                                   sys.exc_info()[1]))
            raise

    def register_test_case_id(test_case):
        tempest_id = test_case.test_data.get('desc')
        test_name = test_case.id()
        if not tempest_id:
            raise AssertionError(
                "No Tempest ID registered for API test %s" % test_name)

        def test_id():
            return test_name + '[id-%s]' % tempest_id

        test_case.id = test_id

    def register_test_suite_ids(test_suite):
        for test_case in test_suite:
            if isinstance(test_case, unittest.TestSuite):
                register_test_suite_ids(test_case)
            else:
                register_test_case_id(test_case)

    cert_validate = not conf.disable_ssl_certificate_validation
    try:
        api_tests = driver.build_tests(test_dir, loader, url=endpoint, host="",
                                       fixture_module=fixtures,
                                       cert_validate=cert_validate,
                                       test_loader_name=__name__)
    except TypeError as ex:
        err_msg = "got an unexpected keyword argument 'cert_validate'"
        if err_msg in six.text_type(ex):
            api_tests = driver.build_tests(test_dir, loader,
                                           url=endpoint, host="",
                                           fixture_module=fixtures,
                                           test_loader_name=__name__)
        else:
            raise

    register_test_suite_ids(api_tests)
    return api_tests



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import select
import socket
import time

from oslo_log import log as logging
import paramiko
import six

from heat_tempest_plugin.common import exceptions

LOG = logging.getLogger(__name__)


class Client(object):

    def __init__(self, host, username, password=None, timeout=300, pkey=None,
                 channel_timeout=10, look_for_keys=False, key_filename=None):
        self.host = host
        self.username = username
        self.password = password
        if isinstance(pkey, six.string_types):
            pkey = paramiko.RSAKey.from_private_key(
                six.moves.cStringIO(str(pkey)))
        self.pkey = pkey
        self.look_for_keys = look_for_keys
        self.key_filename = key_filename
        self.timeout = int(timeout)
        self.channel_timeout = float(channel_timeout)
        self.buf_size = 1024

    def _get_ssh_connection(self, sleep=1.5, backoff=1):
        """Returns an ssh connection to the specified host."""
        bsleep = sleep
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(
            paramiko.AutoAddPolicy())
        _start_time = time.time()
        if self.pkey is not None:
            LOG.info("Creating ssh connection to '%s' as '%s'"
                     " with public key authentication",
                     self.host, self.username)
        else:
            LOG.info("Creating ssh connection to '%s' as '%s'"
                     " with password %s",
                     self.host, self.username, str(self.password))
        attempts = 0
        while True:
            try:
                ssh.connect(self.host, username=self.username,
                            password=self.password,
                            look_for_keys=self.look_for_keys,
                            key_filename=self.key_filename,
                            timeout=self.channel_timeout, pkey=self.pkey)
                LOG.info("ssh connection to %s@%s successfuly created",
                         self.username, self.host)
                return ssh
            except (socket.error,
                    paramiko.SSHException) as e:
                if self._is_timed_out(_start_time):
                    LOG.exception("Failed to establish authenticated ssh"
                                  " connection to %s@%s after %d attempts",
                                  self.username, self.host, attempts)
                    raise exceptions.SSHTimeout(host=self.host,
                                                user=self.username,
                                                password=self.password)
                bsleep += backoff
                attempts += 1
                LOG.warning("Failed to establish authenticated ssh"
                            " connection to %s@%s (%s). Number attempts: %s."
                            " Retry after %d seconds.",
                            self.username, self.host, e, attempts, bsleep)
                time.sleep(bsleep)

    def _is_timed_out(self, start_time):
        return (time.time() - self.timeout) > start_time

    def exec_command(self, cmd):
        """Execute the specified command on the server.

        Note that this method is reading whole command outputs to memory, thus
        shouldn't be used for large outputs.

        :returns: data read from standard output of the command.
        :raises: SSHExecCommandFailed if command returns nonzero
                 status. The exception contains command status stderr content.
        """
        ssh = self._get_ssh_connection()
        transport = ssh.get_transport()
        channel = transport.open_session()
        channel.fileno()  # Register event pipe
        channel.exec_command(cmd)
        channel.shutdown_write()
        out_data = []
        err_data = []
        poll = select.poll()
        poll.register(channel, select.POLLIN)
        start_time = time.time()

        while True:
            ready = poll.poll(self.channel_timeout)
            if not any(ready):
                if not self._is_timed_out(start_time):
                    continue
                raise exceptions.TimeoutException(
                    "Command: '{0}' executed on host '{1}'.".format(
                        cmd, self.host))
            if not ready[0]:  # If there is nothing to read.
                continue
            out_chunk = err_chunk = None
            if channel.recv_ready():
                out_chunk = channel.recv(self.buf_size)
                out_data += out_chunk,
            if channel.recv_stderr_ready():
                err_chunk = channel.recv_stderr(self.buf_size)
                err_data += err_chunk,
            if channel.closed and not err_chunk and not out_chunk:
                break
        exit_status = channel.recv_exit_status()
        if 0 != exit_status:
            raise exceptions.SSHExecCommandFailed(
                command=cmd, exit_status=exit_status,
                strerror=''.join(err_data))
        return ''.join(out_data)

    def test_connection_auth(self):
        """Raises an exception when we can not connect to server via ssh."""
        connection = self._get_ssh_connection()
        connection.close()


class RemoteClient(object):

    # NOTE(afazekas): It should always get an address instead of server
    def __init__(self, server, username, password=None, pkey=None,
                 conf=None):
        self.conf = conf
        ssh_timeout = self.conf.ssh_timeout
        network = self.conf.network_for_ssh
        ip_version = self.conf.ip_version_for_ssh
        ssh_channel_timeout = self.conf.ssh_channel_timeout
        if isinstance(server, six.string_types):
            ip_address = server
        else:
            addresses = server['addresses'][network]
            for address in addresses:
                if address['version'] == ip_version:
                    ip_address = address['addr']
                    break
            else:
                raise exceptions.ServerUnreachable()
        self.ssh_client = Client(ip_address, username, password,
                                 ssh_timeout, pkey=pkey,
                                 channel_timeout=ssh_channel_timeout)

    def exec_command(self, cmd):
        return self.ssh_client.exec_command(cmd)

    def validate_authentication(self):
        """Validate ssh connection and authentication.

        This method raises an Exception when the validation fails.
        """
        self.ssh_client.test_connection_auth()

    def get_partitions(self):
        # Return the contents of /proc/partitions
        command = 'cat /proc/partitions'
        output = self.exec_command(command)
        return output

    def get_boot_time(self):
        cmd = 'cut -f1 -d. /proc/uptime'
        boot_secs = self.exec_command(cmd)
        boot_time = time.time() - int(boot_secs)
        return time.localtime(boot_time)

    def write_to_console(self, message):
        message = re.sub("([$\\`])", "\\\\\\\\\\1", message)
        # usually to /dev/ttyS0
        cmd = 'sudo sh -c "echo \\"%s\\" >/dev/console"' % message
        return self.exec_command(cmd)

    def ping_host(self, host):
        cmd = 'ping -c1 -w1 %s' % host
        return self.exec_command(cmd)

    def get_ip_list(self):
        cmd = "/bin/ip address"
        return self.exec_command(cmd)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


class TemplateAPITest(functional_base.FunctionalTestsBase):
    """This will test the following template calls:

    1. Get the template content for the specific stack
    2. List resource types
    3. Show resource details for OS::Heat::TestResource
    """

    template = {
        'heat_template_version': '2014-10-16',
        'description': 'Test Template APIs',
        'resources': {
            'test1': {
                'type': 'OS::Heat::TestResource',
                'properties': {
                    'update_replace': False,
                    'wait_secs': 0,
                    'value': 'Test1',
                    'fail': False,
                }
            }
        }
    }

    @decorators.idempotent_id('ac6ebc41-bd6a-4df4-80e5-f4b9ae3b5506')
    def test_get_stack_template(self):
        stack_identifier = self.stack_create(
            template=self.template
        )
        template_from_client = self.client.stacks.template(stack_identifier)
        self.assertEqual(self.template, template_from_client)

    @decorators.idempotent_id('9f9a2fc0-f029-4d1f-a2eb-f019b9f75944')
    def test_resource_types(self):
        resource_types = self.client.resource_types.list()
        self.assertTrue(any(resource.resource_type == "OS::Heat::TestResource"
                            for resource in resource_types))

    @decorators.idempotent_id('fafbdcd0-eec3-4e6f-9c88-1e4835d085cf')
    def test_show_resource_template(self):
        resource_details = self.client.resource_types.get(
            resource_type="OS::Heat::TestResource"
        )
        self.assertEqual("OS::Heat::TestResource",
                         resource_details['resource_type'])



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from tempest.lib import decorators

from heat_tempest_plugin.common import exceptions
from heat_tempest_plugin.tests.scenario import scenario_base


class ServerSignalIntegrationTest(scenario_base.ScenarioTestsBase):
    """Test a server in a created network can signal to heat."""

    def _test_server_signal(self, user_data_format='RAW',
                            image=None, flavor=None):
        """Check a server in a created network can signal to heat."""
        parameters = {
            'key_name': self.keypair_name,
            'flavor': flavor,
            'image': image,
            'public_net': self.conf.floating_network_name,
            'timeout': self.conf.build_timeout,
            'user_data_format': user_data_format
        }
        if self.conf.vm_to_heat_api_insecure:
            parameters['wc_extra_args'] = '--insecure'
        # Launch stack
        sid = self.launch_stack(
            template_name="test_server_signal.yaml",
            parameters=parameters,
            expected_status=None
        )

        # Check status of all resources
        for res in ('sg', 'floating_ip', 'network', 'router', 'subnet',
                    'router_interface', 'wait_handle', 'server',
                    'server_floating_ip_assoc'):
            self._wait_for_resource_status(
                sid, res, 'CREATE_COMPLETE')

        server_resource = self.client.resources.get(sid, 'server')
        server_id = server_resource.physical_resource_id
        server = self.compute_client.servers.get(server_id)

        try:
            self._wait_for_resource_status(
                sid, 'wait_condition', 'CREATE_COMPLETE')
        except (exceptions.StackResourceBuildErrorException,
                exceptions.TimeoutException):
            raise
        finally:
            # attempt to log the server console regardless of WaitCondition
            # going to complete. This allows successful and failed cloud-init
            # logs to be compared
            self._log_console_output(servers=[server])

        stack = self.client.stacks.get(sid)

        wc_data = json.loads(
            self._stack_output(stack, 'wc_data'))
        self.assertEqual({'1': 'test complete'}, wc_data)

        server_ip = self._stack_output(stack, 'server_ip')

        # Check that created server is reachable
        if not self._ping_ip_address(server_ip):
            self._log_console_output(servers=[server])
            self.fail(
                "Timed out waiting for %s to become reachable" % server_ip)

    @decorators.idempotent_id('8da0f6cc-60e6-4298-9e54-e1f905c5552a')
    def test_server_signal_userdata_format_raw(self):
        self._test_server_signal(image=self.conf.minimal_image_ref,
                                 flavor=self.conf.minimal_instance_type)

    @decorators.idempotent_id('3d753d42-7c16-4a0e-8f73-875881826626')
    def test_server_signal_userdata_format_software_config(self):
        if not self.conf.image_ref:
            raise self.skipException("No image configured to test")
        self._test_server_signal(user_data_format='SOFTWARE_CONFIG',
                                 image=self.conf.image_ref,
                                 flavor=self.conf.instance_type)



# Copyright 2015
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from tempest import config
from tempest.test_discover import plugins

from heat_tempest_plugin import config as heat_config


class HeatTempestPlugin(plugins.TempestPlugin):
    """A HeatTempestPlugin class

    Provides the basic hooks for an external plugin to provide tempest the
    necessary information to run the plugin.
    """

    def load_tests(self):
        """Provide Load tests information

        Method to return the information necessary to load the tests in the
        plugin.

        :return: a tuple with the first value being the test_dir and the second
                 being the top_level
        :return type: tuple
        """

        base_path = os.path.split(os.path.dirname(
            os.path.abspath(__file__)))[0]
        test_dir = "heat_tempest_plugin"
        full_test_dir = os.path.join(base_path, test_dir)
        return full_test_dir, base_path

    def register_opts(self, conf):
        """Add additional configuration options to tempest.

        This method will be run for the plugin during the register_opts()
        function in tempest.config

        Parameters:
        conf (ConfigOpts): The conf object that can be used to register
        additional options on.
        """
        config.register_opt_group(conf, heat_config.service_available_group,
                                  heat_config.ServiceAvailableGroup)
        config.register_opt_group(conf, heat_config.heat_group,
                                  heat_config.HeatGroup)
        config.register_opt_group(conf, heat_config.heat_features_group,
                                  heat_config.HeatFeaturesGroup)

    def get_opt_lists(self):
        """Get a list of options for sample config generation

        Return option_list: A list of tuples with the group name
                            and options in that group.
        Return type: list
        """

        return [(heat_config.heat_group.name,
                 heat_config.HeatGroup),
                (heat_config.heat_features_group.name,
                 heat_config.HeatFeaturesGroup)]



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

import requests
from tempest.lib import decorators

from heat_tempest_plugin.common import test
from heat_tempest_plugin.tests.scenario import scenario_base


class AutoscalingLoadBalancerTest(scenario_base.ScenarioTestsBase):
    """The class is responsible for testing ASG + LBv1 scenario.

    The very common use case tested is an autoscaling group
    of some web application servers behind a loadbalancer.
    """

    def setUp(self):
        super(AutoscalingLoadBalancerTest, self).setUp()
        self.template_name = 'test_autoscaling_lb_neutron.yaml'
        self.app_server_template_name = 'app_server_neutron.yaml'
        self.webapp_template_name = 'netcat-webapp.yaml'
        if not self.is_network_extension_supported('lbaas'):
            self.skipTest('LBaas v1 extension not available, skipping')

    def check_num_responses(self, url, expected_num, retries=10):
        resp = set()
        for count in range(retries):
            time.sleep(1)
            try:
                r = requests.get(url, verify=self.verify_cert)
            except requests.exceptions.ConnectionError:
                # The LB may not be up yet, let's retry
                continue
            # skip unsuccessful requests
            if r.status_code == 200:
                resp.add(r.text)
        self.assertEqual(expected_num, len(resp))

    @decorators.idempotent_id('48ddbc45-cef6-4640-acd6-7efc281833b9')
    def test_autoscaling_loadbalancer_neutron(self):
        """Check work of AutoScaing and Neutron LBaaS v1 resource in Heat.

        The scenario is the following:
            1. Launch a stack with a load balancer and autoscaling group
               of one server, wait until stack create is complete.
            2. Check that there is only one distinctive response from
               loadbalanced IP.
            3. Signal the scale_up policy, wait until all resources in
               autoscaling group are complete.
            4. Check that now there are two distinctive responses from
               loadbalanced IP.
        """

        parameters = {
            'flavor': self.conf.minimal_instance_type,
            'image': self.conf.minimal_image_ref,
            'net': self.conf.fixed_network_name,
            'subnet': self.conf.fixed_subnet_name,
            'public_net': self.conf.floating_network_name,
            'app_port': 8080,
            'lb_port': 80,
            'timeout': 600
        }
        if self.conf.vm_to_heat_api_insecure:
            parameters['wc_extra_args'] = '--insecure'

        app_server_template = self._load_template(
            __file__, self.app_server_template_name, self.sub_dir
        )
        webapp_template = self._load_template(
            __file__, self.webapp_template_name, self.sub_dir
        )
        files = {'appserver.yaml': app_server_template,
                 'webapp.yaml': webapp_template}
        env = {'resource_registry':
               {'OS::Test::NeutronAppServer': 'appserver.yaml',
                'OS::Test::WebAppConfig': 'webapp.yaml'}}
        # Launch stack
        sid = self.launch_stack(
            template_name=self.template_name,
            parameters=parameters,
            files=files,
            environment=env
        )
        stack = self.client.stacks.get(sid)
        lb_url = self._stack_output(stack, 'lburl')
        # Check number of distinctive responces, must be 1
        self.check_num_responses(lb_url, 1)

        # Signal the scaling hook
        self.client.resources.signal(sid, 'scale_up')

        # Wait for AutoScalingGroup update to finish
        asg = self.client.resources.get(sid, 'asg')
        test.call_until_true(self.conf.build_timeout,
                             self.conf.build_interval,
                             self.check_autoscale_complete,
                             asg.physical_resource_id, 2, sid, 'asg')

        # Check number of distinctive responses, must now be 2
        self.check_num_responses(lb_url, 2)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


class StackEventsTest(functional_base.FunctionalTestsBase):

    template = '''
heat_template_version: 2014-10-16
parameters:
resources:
  test_resource:
    type: OS::Heat::TestResource
    properties:
      value: 'test1'
      fail: False
      update_replace: False
      wait_secs: 0
outputs:
  resource_id:
    description: 'ID of resource'
    value: { get_resource: test_resource }
'''

    def _verify_event_fields(self, event, event_characteristics):
        self.assertIsNotNone(event_characteristics)
        self.assertIsNotNone(event.event_time)
        self.assertIsNotNone(event.links)
        self.assertIsNotNone(event.logical_resource_id)
        self.assertIsNotNone(event.resource_status)
        self.assertIn(event.resource_status, event_characteristics[1])
        self.assertIsNotNone(event.resource_status_reason)
        self.assertIsNotNone(event.id)

    @decorators.idempotent_id('620f4f7c-74f8-48a4-a8b0-d06d0337f133')
    def test_event(self):
        parameters = {}

        test_stack_name = self._stack_rand_name()
        stack_identifier = self.stack_create(
            stack_name=test_stack_name,
            template=self.template,
            parameters=parameters
        )

        expected_status = ['CREATE_IN_PROGRESS', 'CREATE_COMPLETE']
        event_characteristics = {
            test_stack_name: ('OS::Heat::Stack', expected_status),
            'test_resource': ('OS::Heat::TestResource', expected_status)}

        # List stack events
        # API: GET /v1/{tenant_id}/stacks/{stack_name}/{stack_id}/events
        stack_events = self.client.events.list(stack_identifier)

        for stack_event in stack_events:
            # Key on an expected/valid resource name
            self._verify_event_fields(
                stack_event,
                event_characteristics[stack_event.resource_name])

            # Test the event filtering API based on this resource_name
            # /v1/{tenant_id}/stacks/{stack_name}/{stack_id}/resources/{resource_name}/events
            resource_events = self.client.events.list(
                stack_identifier,
                stack_event.resource_name)

            # Resource events are a subset of the original stack event list
            self.assertLess(len(resource_events), len(stack_events))

            # Get the event details for each resource event
            for resource_event in resource_events:
                # A resource_event should be in the original stack event list
                self.assertIn(resource_event, stack_events)
                # Given a filtered list, the resource names should be identical
                self.assertEqual(
                    resource_event.resource_name,
                    stack_event.resource_name)
                # Verify all fields, keying off the resource_name
                self._verify_event_fields(
                    resource_event,
                    event_characteristics[resource_event.resource_name])

                # Exercise the event details API
                # /v1/{tenant_id}/stacks/{stack_name}/{stack_id}/resources/{resource_name}/events/{event_id}
                event_details = self.client.events.get(
                    stack_identifier,
                    resource_event.resource_name,
                    resource_event.id)
                self._verify_event_fields(
                    event_details,
                    event_characteristics[event_details.resource_name])
                # The names should be identical to the non-detailed event
                self.assertEqual(
                    resource_event.resource_name,
                    event_details.resource_name)
                # Verify the extra field in the detail results
                self.assertIsNotNone(event_details.resource_type)
                self.assertEqual(
                    event_characteristics[event_details.resource_name][0],
                    event_details.resource_type)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


class ExternalReferencesTest(functional_base.FunctionalTestsBase):

    TEMPLATE = '''
heat_template_version: 2016-10-14
resources:
  test1:
    type: OS::Heat::TestResource
'''
    TEMPLATE_WITH_EX_REF = '''
heat_template_version: 2016-10-14
resources:
  test1:
    type: OS::Heat::TestResource
    external_id: foobar
outputs:
  str:
    value: {get_resource: test1}
'''

    def _stack_create(self, template):
        self.stack_name = self._stack_rand_name()
        self.stack_identifier = self.stack_create(
            stack_name=self.stack_name,
            template=template,
            files={},
            disable_rollback=True,
            parameters={},
            environment={},
            expected_status='CREATE_COMPLETE'
        )

        expected_resources = {'test1': 'OS::Heat::TestResource'}
        self.assertEqual(expected_resources,
                         self.list_resources(self.stack_identifier))

    @decorators.idempotent_id('45449bad-18ba-4148-82e6-a6bc1e9a9b04')
    def test_create_with_external_ref(self):
        self._stack_create(self.TEMPLATE_WITH_EX_REF)
        stack = self.client.stacks.get(self.stack_identifier)
        self.assertEqual(
            [{'description': 'No description given',
              'output_key': 'str',
              'output_value': 'foobar'}], stack.outputs)

    @decorators.idempotent_id('fb16477c-e981-4ef9-a83b-c0acc162343a')
    def test_update_with_external_ref(self):
        self._stack_create(self.TEMPLATE)

        stack = self.client.stacks.get(self.stack_identifier)
        self.assertEqual([], stack.outputs)

        stack_name = self.stack_identifier.split('/')[0]
        kwargs = {'stack_id': self.stack_identifier, 'stack_name': stack_name,
                  'template': self.TEMPLATE_WITH_EX_REF, 'files': {},
                  'disable_rollback': True, 'parameters': {}, 'environment': {}
                  }
        self.client.stacks.update(**kwargs)
        self._wait_for_stack_status(self.stack_identifier, 'UPDATE_FAILED')

    @decorators.idempotent_id('0ac301c2-b377-49b8-82e2-2458634bc8cf')
    def test_update_stack_contain_external_ref(self):
        self._stack_create(self.TEMPLATE_WITH_EX_REF)

        stack = self.client.stacks.get(self.stack_identifier)
        self.assertEqual(
            [{'description': 'No description given',
              'output_key': 'str',
              'output_value': 'foobar'}], stack.outputs)

        # Update Stack without change external_id

        new_stack_name = self._stack_rand_name()
        kwargs = {'stack_id': self.stack_identifier,
                  'stack_name': new_stack_name,
                  'template': self.TEMPLATE_WITH_EX_REF, 'files': {},
                  'disable_rollback': True, 'parameters': {}, 'environment': {}
                  }
        self.client.stacks.update(**kwargs)

        self._wait_for_stack_status(self.stack_identifier, 'UPDATE_COMPLETE')

        expected_resources = {'test1': 'OS::Heat::TestResource'}
        self.assertEqual(expected_resources,
                         self.list_resources(self.stack_identifier))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random
import re
import subprocess
import time

import fixtures
from heatclient import exc as heat_exceptions
from keystoneauth1 import exceptions as kc_exceptions
from neutronclient.common import exceptions as network_exceptions
from oslo_log import log as logging
from oslo_utils import timeutils
import six
from six.moves import urllib
import testscenarios
import testtools

from heat_tempest_plugin.common import exceptions
from heat_tempest_plugin.common import remote_client
from heat_tempest_plugin.services import clients
from tempest import config

LOG = logging.getLogger(__name__)
_LOG_FORMAT = "%(levelname)8s [%(name)s] %(message)s"
_resource_types = None


def call_until_true(duration, sleep_for, func, *args, **kwargs):
    """Call the function until it returns True or the duration elapsed.

    Call the given function until it returns True (and return True) or
    until the specified duration (in seconds) elapses (and return
    False).

    :param func: A zero argument callable that returns True on success.
    :param duration: The number of seconds for which to attempt a
        successful call of the function.
    :param sleep_for: The number of seconds to sleep after an unsuccessful
                      invocation of the function.
    """
    now = time.time()
    timeout = now + duration
    while now < timeout:
        if func(*args, **kwargs):
            return True
        LOG.debug("Sleeping for %d seconds", sleep_for)
        time.sleep(sleep_for)
        now = time.time()
    return False


def isotime(at):
    if at is None:
        return None
    return at.strftime('%Y-%m-%dT%H:%M:%SZ')


def rand_name(name=''):
    randbits = six.text_type(random.randint(1, 0x7fffffff))
    if name:
        return name + '-' + randbits
    else:
        return randbits


def requires_convergence(test_method):
    '''Decorator for convergence-only tests.

    The decorated test will be skipped when convergence is disabled.
    '''
    plugin = config.CONF.heat_plugin
    convergence_enabled = plugin.convergence_engine_enabled
    skipper = testtools.skipUnless(convergence_enabled,
                                   "Convergence-only tests are disabled")
    return skipper(test_method)


def requires_resource_type(resource_type):
    '''Decorator for tests requiring a resource type.

    The decorated test will be skipped when the resource type is not available.
    '''
    def decorator(test_method):
        conf = getattr(config.CONF, 'heat_plugin', None)
        if not conf or conf.auth_url is None:
            return test_method

        global _resource_types
        if not _resource_types:
            manager = clients.ClientManager(conf)
            obj_rtypes = manager.orchestration_client.resource_types.list()
            _resource_types = list(t.resource_type for t in obj_rtypes)
        rtype_available = resource_type and resource_type in _resource_types
        skipper = testtools.skipUnless(
            rtype_available,
            "%s resource type not available, skipping test." % resource_type)
        return skipper(test_method)
    return decorator


def requires_service_type(service_type):
    '''Decorator for tests requiring a specific service being available.

    The decorated test will be skipped when a service is not available.
    '''
    def decorator(test_method):
        conf = getattr(config.CONF, 'heat_plugin', None)
        if not conf or conf.auth_url is None:
            return test_method

        manager = clients.ClientManager(conf)
        try:
            manager.identity_client.get_endpoint_url(
                service_type, conf.region, conf.endpoint_type)
        except kc_exceptions.EndpointNotFound:
            skipper = testtools.skip(
                "%s service not available, skipping test." % service_type)
            return skipper(test_method)
        else:
            return test_method
    return decorator


def _check_require(group, feature, test_method):
    features_group = getattr(config.CONF, group, None)
    if not features_group:
        return test_method
    feature_enabled = features_group.get(feature, True)
    skipper = testtools.skipUnless(feature_enabled,
                                   "%s - Feature not enabled." % feature)
    return skipper(test_method)


def requires_feature(feature):
    '''Decorator for tests requring specific feature.

    The decorated test will be skipped when a specific feature is disabled.
    '''
    def decorator(test_method):
        return _check_require('heat_features_enabled', feature, test_method)
    return decorator


def requires_service_feature(service, feature):
    '''Decorator for tests requring specific service feature enabled in tempest.

    The decorated test will be skipped when a specific feature is disabled.
    '''
    def decorator(test_method):
        group = service + '_feature_enabled'
        return _check_require(group, feature, test_method)
    return decorator


class HeatIntegrationTest(testtools.testcase.WithAttributes,
                          testscenarios.WithScenarios,
                          testtools.TestCase):

    def setUp(self):
        super(HeatIntegrationTest, self).setUp()

        self.conf = config.CONF.heat_plugin

        self.assertIsNotNone(self.conf.auth_url,
                             'No auth_url configured')
        self.assertIsNotNone(self.conf.username,
                             'No username configured')
        self.assertIsNotNone(self.conf.password,
                             'No password configured')
        self.setup_clients(self.conf)
        self.useFixture(fixtures.FakeLogger(format=_LOG_FORMAT))
        if self.conf.disable_ssl_certificate_validation:
            self.verify_cert = False
        else:
            self.verify_cert = self.conf.ca_file or True

    def setup_clients(self, conf, admin_credentials=False):
        self.manager = clients.ClientManager(conf, admin_credentials)
        self.identity_client = self.manager.identity_client
        self.orchestration_client = self.manager.orchestration_client
        self.compute_client = self.manager.compute_client
        self.network_client = self.manager.network_client
        self.volume_client = self.manager.volume_client
        self.object_client = self.manager.object_client
        self.metric_client = self.manager.metric_client

        self.client = self.orchestration_client

    def setup_clients_for_admin(self):
        self.setup_clients(self.conf, True)

    def get_remote_client(self, server_or_ip, username, private_key=None):
        if isinstance(server_or_ip, six.string_types):
            ip = server_or_ip
        else:
            network_name_for_ssh = self.conf.network_for_ssh
            ip = server_or_ip.networks[network_name_for_ssh][0]
        if private_key is None:
            private_key = self.keypair.private_key
        linux_client = remote_client.RemoteClient(ip, username,
                                                  pkey=private_key,
                                                  conf=self.conf)
        try:
            linux_client.validate_authentication()
        except exceptions.SSHTimeout:
            LOG.exception('ssh connection to %s failed', ip)
            raise

        return linux_client

    def check_connectivity(self, check_ip):
        def try_connect(ip):
            try:
                urllib.request.urlopen('http://%s/' % ip)
                return True
            except IOError:
                return False

        timeout = self.conf.connectivity_timeout
        elapsed_time = 0
        while not try_connect(check_ip):
            time.sleep(10)
            elapsed_time += 10
            if elapsed_time > timeout:
                raise exceptions.TimeoutException()

    def _log_console_output(self, servers=None):
        if not servers:
            servers = self.compute_client.servers.list()
        for server in servers:
            LOG.info('Console output for %s', server.id)
            LOG.info(server.get_console_output())

    def create_keypair(self, client=None, name=None):
        if client is None:
            client = self.compute_client
        if name is None:
            name = rand_name('heat-keypair')
        keypair = client.keypairs.create(name)
        self.assertEqual(keypair.name, name)

        def delete_keypair():
            keypair.delete()

        self.addCleanup(delete_keypair)
        return keypair

    def assign_keypair(self):
        if self.conf.keypair_name:
            self.keypair = None
            self.keypair_name = self.conf.keypair_name
        else:
            self.keypair = self.create_keypair()
            self.keypair_name = self.keypair.id

    @classmethod
    def _stack_rand_name(cls):
        return rand_name(cls.__name__)

    def _get_network(self, net_name=None):
        if net_name is None:
            net_name = self.conf.fixed_network_name
        networks = self.network_client.list_networks()
        for net in networks['networks']:
            if net['name'] == net_name:
                return net

    def is_network_extension_supported(self, extension_alias):
        try:
            self.network_client.show_extension(extension_alias)
        except network_exceptions.NeutronClientException:
            return False
        return True

    @staticmethod
    def _stack_output(stack, output_key, validate_errors=True):
        """Return a stack output value for a given key."""
        value = None
        for o in stack.outputs:
            if validate_errors and 'output_error' in o:
                # scan for errors in the stack output.
                raise ValueError(
                    'Unexpected output errors in %s : %s' % (
                        output_key, o['output_error']))
            if o['output_key'] == output_key:
                value = o['output_value']
        return value

    def _ping_ip_address(self, ip_address, should_succeed=True):
        cmd = ['ping', '-c1', '-w1', ip_address]

        def ping():
            proc = subprocess.Popen(cmd,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
            proc.wait()
            return (proc.returncode == 0) == should_succeed

        return call_until_true(
            self.conf.build_timeout, 1, ping)

    def _wait_for_all_resource_status(self, stack_identifier,
                                      status, failure_pattern='^.*_FAILED$',
                                      success_on_not_found=False):
        for res in self.client.resources.list(stack_identifier):
            self._wait_for_resource_status(
                stack_identifier, res.resource_name,
                status, failure_pattern=failure_pattern,
                success_on_not_found=success_on_not_found)

    def _wait_for_resource_status(self, stack_identifier, resource_name,
                                  status, failure_pattern='^.*_FAILED$',
                                  success_on_not_found=False):
        """Waits for a Resource to reach a given status."""
        fail_regexp = re.compile(failure_pattern)
        build_timeout = self.conf.build_timeout
        build_interval = self.conf.build_interval

        start = timeutils.utcnow()
        while timeutils.delta_seconds(start,
                                      timeutils.utcnow()) < build_timeout:
            try:
                res = self.client.resources.get(
                    stack_identifier, resource_name)
            except heat_exceptions.HTTPNotFound:
                if success_on_not_found:
                    return
                # ignore this, as the resource may not have
                # been created yet
            else:
                if res.resource_status == status:
                    return
                wait_for_action = status.split('_')[0]
                resource_action = res.resource_status.split('_')[0]
                if (resource_action == wait_for_action and
                        fail_regexp.search(res.resource_status)):
                    raise exceptions.StackResourceBuildErrorException(
                        resource_name=res.resource_name,
                        stack_identifier=stack_identifier,
                        resource_status=res.resource_status,
                        resource_status_reason=res.resource_status_reason)
            time.sleep(build_interval)

        message = ('Resource %s failed to reach %s status within '
                   'the required time (%s s).' %
                   (resource_name, status, build_timeout))
        raise exceptions.TimeoutException(message)

    def verify_resource_status(self, stack_identifier, resource_name,
                               status='CREATE_COMPLETE'):
        try:
            res = self.client.resources.get(stack_identifier, resource_name)
        except heat_exceptions.HTTPNotFound:
            return False
        return res.resource_status == status

    def _verify_status(self, stack, stack_identifier, status,
                       fail_regexp, is_action_cancelled=False):
        if stack.stack_status == status:
            if status == 'DELETE_COMPLETE' and stack.deletion_time is None:
                # Wait for deleted_time to be filled, so that we have more
                # confidence the operation is finished.
                return False
            else:
                return True

        wait_for_action = status.split('_')[0]
        if (stack.action == wait_for_action and
                fail_regexp.search(stack.stack_status)):
            raise exceptions.StackBuildErrorException(
                stack_identifier=stack_identifier,
                stack_status=stack.stack_status,
                stack_status_reason=stack.stack_status_reason)

        return False

    def _wait_for_stack_status(self, stack_identifier, status,
                               failure_pattern=None,
                               success_on_not_found=False,
                               signal_required=False,
                               resources_to_signal=None,
                               is_action_cancelled=False):
        """Waits for a Stack to reach a given status.

        Note this compares the full $action_$status, e.g
        CREATE_COMPLETE, not just COMPLETE which is exposed
        via the status property of Stack in heatclient
        """
        if failure_pattern:
            fail_regexp = re.compile(failure_pattern)
        elif 'FAILED' in status:
            # If we're looking for e.g CREATE_FAILED, COMPLETE is unexpected.
            fail_regexp = re.compile('^.*_COMPLETE$')
        else:
            fail_regexp = re.compile('^.*_FAILED$')
        build_timeout = self.conf.build_timeout
        build_interval = self.conf.build_interval

        start = timeutils.utcnow()
        while timeutils.delta_seconds(start,
                                      timeutils.utcnow()) < build_timeout:
            try:
                stack = self.client.stacks.get(stack_identifier,
                                               resolve_outputs=False)
            except heat_exceptions.HTTPNotFound:
                if success_on_not_found:
                    return
                # ignore this, as the resource may not have
                # been created yet
            else:
                if self._verify_status(stack, stack_identifier, status,
                                       fail_regexp, is_action_cancelled):
                    return
            if signal_required:
                self.signal_resources(resources_to_signal)
            time.sleep(build_interval)

        message = ('Stack %s failed to reach %s status within '
                   'the required time (%s s).' %
                   (stack_identifier, status, build_timeout))
        raise exceptions.TimeoutException(message)

    def _stack_delete(self, stack_identifier):
        try:
            self._handle_in_progress(self.client.stacks.delete,
                                     stack_identifier)
        except heat_exceptions.HTTPNotFound:
            pass
        self._wait_for_stack_status(
            stack_identifier, 'DELETE_COMPLETE',
            success_on_not_found=True)

    def _handle_in_progress(self, fn, *args, **kwargs):
        build_timeout = self.conf.build_timeout
        build_interval = self.conf.build_interval
        start = timeutils.utcnow()
        while timeutils.delta_seconds(start,
                                      timeutils.utcnow()) < build_timeout:
            try:
                fn(*args, **kwargs)
            except heat_exceptions.HTTPConflict as ex:
                # FIXME(sirushtim): Wait a little for the stack lock to be
                # released and hopefully, the stack should be usable again.
                if ex.error['error']['type'] != 'ActionInProgress':
                    raise ex

                time.sleep(build_interval)
            else:
                break

    def update_stack(self, stack_identifier, template=None, environment=None,
                     files=None, parameters=None, tags=None,
                     expected_status='UPDATE_COMPLETE',
                     disable_rollback=True,
                     existing=False):
        env = environment or {}
        env_files = files or {}
        parameters = parameters or {}

        self._handle_in_progress(
            self.client.stacks.update,
            stack_id=stack_identifier,
            template=template,
            files=env_files,
            disable_rollback=disable_rollback,
            parameters=parameters,
            environment=env,
            tags=tags,
            existing=existing)

        kwargs = {'stack_identifier': stack_identifier,
                  'status': expected_status}
        if expected_status in ['ROLLBACK_COMPLETE']:
            # To trigger rollback you would intentionally fail the stack
            # Hence check for rollback failures
            kwargs['failure_pattern'] = '^ROLLBACK_FAILED$'

        self._wait_for_stack_status(**kwargs)

    def cancel_update_stack(self, stack_identifier, rollback=True,
                            expected_status='ROLLBACK_COMPLETE'):

        stack_name = stack_identifier.split('/')[0]

        if rollback:
            self.client.actions.cancel_update(stack_name)
        else:
            self.client.actions.cancel_without_rollback(stack_name)

        kwargs = {'stack_identifier': stack_identifier,
                  'status': expected_status}
        if expected_status == 'UPDATE_FAILED':
            kwargs['is_action_cancelled'] = True

        if expected_status in ['ROLLBACK_COMPLETE']:
            # To trigger rollback you would intentionally fail the stack
            # Hence check for rollback failures
            kwargs['failure_pattern'] = '^ROLLBACK_FAILED$'

        self._wait_for_stack_status(**kwargs)

    def preview_update_stack(self, stack_identifier, template,
                             environment=None, files=None, parameters=None,
                             tags=None, disable_rollback=True,
                             show_nested=False):
        env = environment or {}
        env_files = files or {}
        parameters = parameters or {}

        return self.client.stacks.preview_update(
            stack_id=stack_identifier,
            template=template,
            files=env_files,
            disable_rollback=disable_rollback,
            parameters=parameters,
            environment=env,
            tags=tags,
            show_nested=show_nested
        )

    def assert_resource_is_a_stack(self, stack_identifier, res_name,
                                   wait=False):
        build_timeout = self.conf.build_timeout
        build_interval = self.conf.build_interval
        start = timeutils.utcnow()
        while timeutils.delta_seconds(start,
                                      timeutils.utcnow()) < build_timeout:
            time.sleep(build_interval)
            try:
                nested_identifier = self._get_nested_identifier(
                    stack_identifier, res_name)
            except Exception:
                # We may have to wait, if the create is in-progress
                if wait:
                    time.sleep(build_interval)
                else:
                    raise
            else:
                return nested_identifier

    def _get_nested_identifier(self, stack_identifier, res_name):
        rsrc = self.client.resources.get(stack_identifier, res_name)
        nested_link = [l for l in rsrc.links if l['rel'] == 'nested']
        nested_href = nested_link[0]['href']
        nested_id = nested_href.split('/')[-1]
        nested_identifier = '/'.join(nested_href.split('/')[-2:])
        self.assertEqual(rsrc.physical_resource_id, nested_id)

        nested_stack = self.client.stacks.get(nested_id, resolve_outputs=False)
        nested_identifier2 = '%s/%s' % (nested_stack.stack_name,
                                        nested_stack.id)
        self.assertEqual(nested_identifier, nested_identifier2)
        parent_id = stack_identifier.split("/")[-1]
        self.assertEqual(parent_id, nested_stack.parent)
        return nested_identifier

    def group_nested_identifier(self, stack_identifier,
                                group_name):
        # Get the nested stack identifier from a group resource
        rsrc = self.client.resources.get(stack_identifier, group_name)
        physical_resource_id = rsrc.physical_resource_id

        nested_stack = self.client.stacks.get(physical_resource_id,
                                              resolve_outputs=False)
        nested_identifier = '%s/%s' % (nested_stack.stack_name,
                                       nested_stack.id)
        parent_id = stack_identifier.split("/")[-1]
        self.assertEqual(parent_id, nested_stack.parent)
        return nested_identifier

    def list_group_resources(self, stack_identifier,
                             group_name, minimal=True):
        nested_identifier = self.group_nested_identifier(stack_identifier,
                                                         group_name)
        if minimal:
            return self.list_resources(nested_identifier)
        return self.client.resources.list(nested_identifier)

    def list_resources(self, stack_identifier):
        resources = self.client.resources.list(stack_identifier)
        return dict((r.resource_name, r.resource_type) for r in resources)

    def get_resource_stack_id(self, r):
        stack_link = [l for l in r.links if l.get('rel') == 'stack'][0]
        return stack_link['href'].split("/")[-1]

    def get_physical_resource_id(self, stack_identifier, resource_name):
        try:
            resource = self.client.resources.get(
                stack_identifier, resource_name)
            return resource.physical_resource_id
        except Exception:
            raise Exception('Resource (%s) not found in stack (%s)!' %
                            (stack_identifier, resource_name))

    def get_stack_output(self, stack_identifier, output_key,
                         validate_errors=True):
        stack = self.client.stacks.get(stack_identifier)
        return self._stack_output(stack, output_key, validate_errors)

    def check_input_values(self, group_resources, key, value):
        # Check inputs for deployment and derived config
        for r in group_resources:
            d = self.client.software_deployments.get(
                r.physical_resource_id)
            self.assertEqual({key: value}, d.input_values)
            c = self.client.software_configs.get(
                d.config_id)
            foo_input_c = [i for i in c.inputs if i.get('name') == key][0]
            self.assertEqual(value, foo_input_c.get('value'))

    def signal_resources(self, resources):
        # Signal all IN_PROGRESS resources
        for r in resources:
            if 'IN_PROGRESS' in r.resource_status:
                stack_id = self.get_resource_stack_id(r)
                self.client.resources.signal(stack_id, r.resource_name)

    def stack_create(self, stack_name=None, template=None, files=None,
                     parameters=None, environment=None, tags=None,
                     expected_status='CREATE_COMPLETE',
                     disable_rollback=True, enable_cleanup=True,
                     environment_files=None, timeout=None):
        name = stack_name or self._stack_rand_name()
        templ = template or self.template
        templ_files = files or {}
        params = parameters or {}
        env = environment or {}
        timeout_mins = timeout or self.conf.build_timeout
        self.client.stacks.create(
            stack_name=name,
            template=templ,
            files=templ_files,
            disable_rollback=disable_rollback,
            parameters=params,
            environment=env,
            tags=tags,
            environment_files=environment_files,
            timeout_mins=timeout_mins
        )
        if enable_cleanup:
            self.addCleanup(self._stack_delete, name)

        stack = self.client.stacks.get(name, resolve_outputs=False)
        stack_identifier = '%s/%s' % (name, stack.id)
        kwargs = {'stack_identifier': stack_identifier,
                  'status': expected_status}
        if expected_status:
            if expected_status in ['ROLLBACK_COMPLETE']:
                # To trigger rollback you would intentionally fail the stack
                # Hence check for rollback failures
                kwargs['failure_pattern'] = '^ROLLBACK_FAILED$'
            self._wait_for_stack_status(**kwargs)
        return stack_identifier

    def stack_adopt(self, stack_name=None, files=None,
                    parameters=None, environment=None, adopt_data=None,
                    wait_for_status='ADOPT_COMPLETE'):
        if (self.conf.skip_test_stack_action_list and
                'ADOPT' in self.conf.skip_test_stack_action_list):
            self.skipTest('Testing Stack adopt disabled in conf, skipping')
        name = stack_name or self._stack_rand_name()
        templ_files = files or {}
        params = parameters or {}
        env = environment or {}
        self.client.stacks.create(
            stack_name=name,
            files=templ_files,
            disable_rollback=True,
            parameters=params,
            environment=env,
            adopt_stack_data=adopt_data,
        )
        self.addCleanup(self._stack_delete, name)
        stack = self.client.stacks.get(name, resolve_outputs=False)
        stack_identifier = '%s/%s' % (name, stack.id)
        self._wait_for_stack_status(stack_identifier, wait_for_status)
        return stack_identifier

    def stack_abandon(self, stack_id):
        if (self.conf.skip_test_stack_action_list and
                'ABANDON' in self.conf.skip_test_stack_action_list):
            self.addCleanup(self._stack_delete, stack_id)
            self.skipTest('Testing Stack abandon disabled in conf, skipping')
        info = self.client.stacks.abandon(stack_id=stack_id)
        return info

    def stack_snapshot(self, stack_id,
                       wait_for_status='SNAPSHOT_COMPLETE'):
        snapshot = self.client.stacks.snapshot(stack_id=stack_id)
        self._wait_for_stack_status(stack_id, wait_for_status)
        return snapshot['id']

    def stack_restore(self, stack_id, snapshot_id,
                      wait_for_status='RESTORE_COMPLETE'):
        self.client.stacks.restore(stack_id, snapshot_id)
        self._wait_for_stack_status(stack_id, wait_for_status)

    def stack_suspend(self, stack_identifier):
        if (self.conf.skip_test_stack_action_list and
                'SUSPEND' in self.conf.skip_test_stack_action_list):
            self.addCleanup(self._stack_delete, stack_identifier)
            self.skipTest('Testing Stack suspend disabled in conf, skipping')
        self._handle_in_progress(self.client.actions.suspend, stack_identifier)
        # improve debugging by first checking the resource's state.
        self._wait_for_all_resource_status(stack_identifier,
                                           'SUSPEND_COMPLETE')
        self._wait_for_stack_status(stack_identifier, 'SUSPEND_COMPLETE')

    def stack_resume(self, stack_identifier):
        if (self.conf.skip_test_stack_action_list and
                'RESUME' in self.conf.skip_test_stack_action_list):
            self.addCleanup(self._stack_delete, stack_identifier)
            self.skipTest('Testing Stack resume disabled in conf, skipping')
        self._handle_in_progress(self.client.actions.resume, stack_identifier)
        # improve debugging by first checking the resource's state.
        self._wait_for_all_resource_status(stack_identifier,
                                           'RESUME_COMPLETE')
        self._wait_for_stack_status(stack_identifier, 'RESUME_COMPLETE')

    def wait_for_event_with_reason(self, stack_identifier, reason,
                                   rsrc_name=None, num_expected=1):
        build_timeout = self.conf.build_timeout
        build_interval = self.conf.build_interval
        start = timeutils.utcnow()
        while timeutils.delta_seconds(start,
                                      timeutils.utcnow()) < build_timeout:
            try:
                rsrc_events = self.client.events.list(stack_identifier,
                                                      resource_name=rsrc_name)
            except heat_exceptions.HTTPNotFound:
                LOG.debug("No events yet found for %s", rsrc_name)
            else:
                matched = [e for e in rsrc_events
                           if e.resource_status_reason == reason]
                if len(matched) == num_expected:
                    return matched
            time.sleep(build_interval)

    def check_autoscale_complete(self, stack_id, expected_num, parent_stack,
                                 group_name):
        res_list = self.client.resources.list(stack_id)
        all_res_complete = all(res.resource_status in ('UPDATE_COMPLETE',
                                                       'CREATE_COMPLETE')
                               for res in res_list)
        all_res = len(res_list) == expected_num
        if all_res and all_res_complete:
            metadata = self.client.resources.metadata(parent_stack, group_name)
            return not metadata.get('scaling_in_progress')
        return False



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


test_template = '''
heat_template_version: 2015-04-30
description: Test template to create port wit ip_address.
parameters:
  mac:
    type: string
    default: 00-00-00-00-BB-BB
resources:
  net:
    type: OS::Neutron::Net
  subnet:
    type: OS::Neutron::Subnet
    properties:
      enable_dhcp: false
      network: { get_resource: net }
      cidr: 11.11.11.0/24
  port:
    type: OS::Neutron::Port
    properties:
      network: {get_resource: net}
      mac_address: {get_param: mac}
      fixed_ips:
        - subnet: {get_resource: subnet}
          ip_address: 11.11.11.11
  test:
    depends_on: port
    type: OS::Heat::TestResource
    properties:
      value: Test1
      fail: False
outputs:
  port_ip:
    value: {get_attr: [port, fixed_ips, 0, ip_address]}
  mac_address:
    value: {get_attr: [port, mac_address]}
'''


class UpdatePortTest(functional_base.FunctionalTestsBase):

    def get_port_id_and_outputs(self, stack_identifier):
        resources = self.client.resources.list(stack_identifier)
        port_id = [res.physical_resource_id for res in resources
                   if res.resource_name == 'port']
        stack = self.client.stacks.get(stack_identifier)
        port_ip = self._stack_output(stack, 'port_ip')
        port_mac = self._stack_output(stack, 'mac_address')
        return port_id[0], port_ip, port_mac

    @decorators.idempotent_id('8e37933b-345c-4ce5-a523-e67bdf198d73')
    def test_update_remove_ip(self):
        # create with defined ip_address
        stack_identifier = self.stack_create(template=test_template)
        _id, _ip, _mac = self.get_port_id_and_outputs(stack_identifier)

        # remove ip_address property and update stack
        templ_no_ip = test_template.replace('ip_address: 11.11.11.11', '')
        self.update_stack(stack_identifier, templ_no_ip)

        new_id, new_ip, new_mac = self.get_port_id_and_outputs(
            stack_identifier)
        # port should be updated with the same id
        self.assertEqual(_id, new_id)
        self.assertEqual(_mac, new_mac)

    @decorators.idempotent_id('df1bf0fb-115e-42f3-b6cc-e12b5d506c98')
    def test_update_with_mac_address(self):
        if not self.conf.admin_username or not self.conf.admin_password:
            self.skipTest('No admin creds found, skipping')

        # Setup admin clients for updating mac_address
        self.setup_clients_for_admin()

        # Create with default mac_address and defined ip_address
        stack_identifier = self.stack_create(template=test_template)
        _id, _ip, _mac = self.get_port_id_and_outputs(stack_identifier)

        # Update with another 'mac' parameter
        parameters = {'mac': '00-00-00-00-AA-AA'}
        self.update_stack(stack_identifier, test_template,
                          parameters=parameters)

        new_id, new_ip, new_mac = self.get_port_id_and_outputs(
            stack_identifier)
        # mac_address should be different
        self.assertEqual(_ip, new_ip)
        self.assertNotEqual(_mac, new_mac)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import yaml

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


class HooksTest(functional_base.FunctionalTestsBase):

    def setUp(self):
        super(HooksTest, self).setUp()
        self.template = {'heat_template_version': '2014-10-16',
                         'resources': {
                             'foo_step1': {'type': 'OS::Heat::RandomString'},
                             'foo_step2': {'type': 'OS::Heat::RandomString',
                                           'depends_on': 'foo_step1'},
                             'foo_step3': {'type': 'OS::Heat::RandomString',
                                           'depends_on': 'foo_step2'}}}

    @decorators.idempotent_id('f14cbe86-0392-474a-9f28-bb561745698d')
    def test_hook_pre_create(self):
        env = {'resource_registry':
               {'resources':
                {'foo_step2':
                 {'hooks': 'pre-create'}}}}
        # Note we don't wait for CREATE_COMPLETE, because we need to
        # signal to clear the hook before create will complete
        stack_identifier = self.stack_create(
            template=self.template,
            environment=env,
            expected_status='CREATE_IN_PROGRESS')
        self._wait_for_resource_status(
            stack_identifier, 'foo_step1', 'CREATE_COMPLETE')
        self._wait_for_resource_status(
            stack_identifier, 'foo_step2', 'INIT_COMPLETE')
        ev = self.wait_for_event_with_reason(
            stack_identifier,
            reason='CREATE paused until Hook pre-create is cleared',
            rsrc_name='foo_step2')
        self.assertEqual('INIT_COMPLETE', ev[0].resource_status)
        self.client.resources.signal(stack_identifier, 'foo_step2',
                                     data={'unset_hook': 'pre-create'})
        ev = self.wait_for_event_with_reason(
            stack_identifier,
            reason='Hook pre-create is cleared',
            rsrc_name='foo_step2')
        self.assertEqual('INIT_COMPLETE', ev[0].resource_status)
        self._wait_for_resource_status(
            stack_identifier, 'foo_step2', 'CREATE_COMPLETE')
        self._wait_for_stack_status(stack_identifier, 'CREATE_COMPLETE')

    @decorators.idempotent_id('75009076-96eb-43af-b4df-f002704737d2')
    def test_hook_pre_update_nochange(self):
        env = {'resource_registry':
               {'resources':
                {'foo_step2':
                 {'hooks': 'pre-update'}}}}
        stack_identifier = self.stack_create(
            template=self.template,
            environment=env)
        res_before = self.client.resources.get(stack_identifier, 'foo_step2')
        # Note we don't wait for UPDATE_COMPLETE, because we need to
        # signal to clear the hook before update will complete
        self.update_stack(
            stack_identifier,
            template=self.template,
            environment=env,
            expected_status='UPDATE_IN_PROGRESS')

        # Note when a hook is specified, the resource status doesn't change
        # when we hit the hook, so we look for the event, then assert the
        # state is unchanged.
        self._wait_for_resource_status(
            stack_identifier, 'foo_step2', 'CREATE_COMPLETE')
        ev = self.wait_for_event_with_reason(
            stack_identifier,
            reason='UPDATE paused until Hook pre-update is cleared',
            rsrc_name='foo_step2')
        self.assertEqual('CREATE_COMPLETE', ev[0].resource_status)
        self.client.resources.signal(stack_identifier, 'foo_step2',
                                     data={'unset_hook': 'pre-update'})
        ev = self.wait_for_event_with_reason(
            stack_identifier,
            reason='Hook pre-update is cleared',
            rsrc_name='foo_step2')
        self.assertEqual('CREATE_COMPLETE', ev[0].resource_status)
        self._wait_for_resource_status(
            stack_identifier, 'foo_step2', 'CREATE_COMPLETE')
        self._wait_for_stack_status(stack_identifier, 'UPDATE_COMPLETE')
        res_after = self.client.resources.get(stack_identifier, 'foo_step2')
        self.assertEqual(res_before.physical_resource_id,
                         res_after.physical_resource_id)

    @decorators.idempotent_id('9f03d1e0-a2d2-42fc-a9c6-47db05b9c8ed')
    def test_hook_pre_update_replace(self):
        env = {'resource_registry':
               {'resources':
                {'foo_step2':
                 {'hooks': 'pre-update'}}}}
        stack_identifier = self.stack_create(
            template=self.template,
            environment=env)
        res_before = self.client.resources.get(stack_identifier, 'foo_step2')
        # Note we don't wait for UPDATE_COMPLETE, because we need to
        # signal to clear the hook before update will complete
        self.template['resources']['foo_step2']['properties'] = {'length': 10}
        self.update_stack(
            stack_identifier,
            template=self.template,
            environment=env,
            expected_status='UPDATE_IN_PROGRESS')

        # Note when a hook is specified, the resource status doesn't change
        # when we hit the hook, so we look for the event, then assert the
        # state is unchanged.
        self._wait_for_resource_status(
            stack_identifier, 'foo_step2', 'CREATE_COMPLETE')
        ev = self.wait_for_event_with_reason(
            stack_identifier,
            reason='UPDATE paused until Hook pre-update is cleared',
            rsrc_name='foo_step2')
        self.assertEqual('CREATE_COMPLETE', ev[0].resource_status)
        self.client.resources.signal(stack_identifier, 'foo_step2',
                                     data={'unset_hook': 'pre-update'})
        ev = self.wait_for_event_with_reason(
            stack_identifier,
            reason='Hook pre-update is cleared',
            rsrc_name='foo_step2')
        self.assertEqual('CREATE_COMPLETE', ev[0].resource_status)
        self._wait_for_resource_status(
            stack_identifier, 'foo_step2', 'CREATE_COMPLETE')
        self._wait_for_stack_status(stack_identifier, 'UPDATE_COMPLETE')
        res_after = self.client.resources.get(stack_identifier, 'foo_step2')
        self.assertNotEqual(res_before.physical_resource_id,
                            res_after.physical_resource_id)

    @decorators.idempotent_id('15c4f97d-5e06-40c6-83af-0e57d5afaff9')
    def test_hook_pre_update_in_place(self):
        env = {'resource_registry':
               {'resources':
                {'rg':
                 {'hooks': 'pre-update'}}}}
        template = {'heat_template_version': '2014-10-16',
                    'resources': {
                        'rg': {
                            'type': 'OS::Heat::ResourceGroup',
                            'properties': {
                                'count': 1,
                                'resource_def': {
                                    'type': 'OS::Heat::RandomString'}}}}}
        # Note we don't wait for CREATE_COMPLETE, because we need to
        # signal to clear the hook before create will complete
        stack_identifier = self.stack_create(
            template=template,
            environment=env)
        res_before = self.client.resources.get(stack_identifier, 'rg')
        template['resources']['rg']['properties']['count'] = 2
        self.update_stack(
            stack_identifier,
            template=template,
            environment=env,
            expected_status='UPDATE_IN_PROGRESS')

        # Note when a hook is specified, the resource status doesn't change
        # when we hit the hook, so we look for the event, then assert the
        # state is unchanged.
        self._wait_for_resource_status(
            stack_identifier, 'rg', 'CREATE_COMPLETE')
        ev = self.wait_for_event_with_reason(
            stack_identifier,
            reason='UPDATE paused until Hook pre-update is cleared',
            rsrc_name='rg')
        self.assertEqual('CREATE_COMPLETE', ev[0].resource_status)
        self.client.resources.signal(stack_identifier, 'rg',
                                     data={'unset_hook': 'pre-update'})

        ev = self.wait_for_event_with_reason(
            stack_identifier,
            reason='Hook pre-update is cleared',
            rsrc_name='rg')
        self.assertEqual('CREATE_COMPLETE', ev[0].resource_status)
        self._wait_for_stack_status(stack_identifier, 'UPDATE_COMPLETE')
        res_after = self.client.resources.get(stack_identifier, 'rg')
        self.assertEqual(res_before.physical_resource_id,
                         res_after.physical_resource_id)

    @decorators.idempotent_id('7d96b5f4-4aba-493d-b098-4beea4654a2b')
    def test_hook_pre_create_nested(self):
        files = {'nested.yaml': yaml.safe_dump(self.template)}
        env = {'resource_registry':
               {'resources':
                {'nested':
                 {'foo_step2':
                  {'hooks': 'pre-create'}}}}}
        template = {'heat_template_version': '2014-10-16',
                    'resources': {
                        'nested': {'type': 'nested.yaml'}}}
        # Note we don't wait for CREATE_COMPLETE, because we need to
        # signal to clear the hook before create will complete
        stack_identifier = self.stack_create(
            template=template,
            environment=env,
            files=files,
            expected_status='CREATE_IN_PROGRESS')
        self._wait_for_resource_status(stack_identifier, 'nested',
                                       'CREATE_IN_PROGRESS')
        nested_identifier = self.assert_resource_is_a_stack(
            stack_identifier, 'nested', wait=True)
        self._wait_for_resource_status(
            nested_identifier, 'foo_step1', 'CREATE_COMPLETE')
        self._wait_for_resource_status(
            nested_identifier, 'foo_step2', 'INIT_COMPLETE')
        ev = self.wait_for_event_with_reason(
            nested_identifier,
            reason='CREATE paused until Hook pre-create is cleared',
            rsrc_name='foo_step2')
        self.assertEqual('INIT_COMPLETE', ev[0].resource_status)
        self.client.resources.signal(nested_identifier, 'foo_step2',
                                     data={'unset_hook': 'pre-create'})
        ev = self.wait_for_event_with_reason(
            nested_identifier,
            reason='Hook pre-create is cleared',
            rsrc_name='foo_step2')
        self.assertEqual('INIT_COMPLETE', ev[0].resource_status)
        self._wait_for_resource_status(
            nested_identifier, 'foo_step2', 'CREATE_COMPLETE')
        self._wait_for_stack_status(stack_identifier, 'CREATE_COMPLETE')

    @decorators.idempotent_id('cffc0783-3add-4f0c-ac66-03c90f6feeb0')
    def test_hook_pre_create_wildcard(self):
        env = {'resource_registry':
               {'resources':
                {'foo_*':
                 {'hooks': 'pre-create'}}}}
        # Note we don't wait for CREATE_COMPLETE, because we need to
        # signal to clear the hook before create will complete
        stack_identifier = self.stack_create(
            template=self.template,
            environment=env,
            expected_status='CREATE_IN_PROGRESS')
        self._wait_for_resource_status(
            stack_identifier, 'foo_step1', 'INIT_COMPLETE')
        self.wait_for_event_with_reason(
            stack_identifier,
            reason='CREATE paused until Hook pre-create is cleared',
            rsrc_name='foo_step1')
        self.client.resources.signal(stack_identifier, 'foo_step1',
                                     data={'unset_hook': 'pre-create'})
        self.wait_for_event_with_reason(
            stack_identifier,
            reason='Hook pre-create is cleared',
            rsrc_name='foo_step1')
        self._wait_for_resource_status(
            stack_identifier, 'foo_step2', 'INIT_COMPLETE')
        self.wait_for_event_with_reason(
            stack_identifier,
            reason='CREATE paused until Hook pre-create is cleared',
            rsrc_name='foo_step2')
        self.client.resources.signal(stack_identifier, 'foo_step2',
                                     data={'unset_hook': 'pre-create'})
        self.wait_for_event_with_reason(
            stack_identifier,
            reason='Hook pre-create is cleared',
            rsrc_name='foo_step2')
        self._wait_for_resource_status(
            stack_identifier, 'foo_step3', 'INIT_COMPLETE')
        self.wait_for_event_with_reason(
            stack_identifier,
            reason='CREATE paused until Hook pre-create is cleared',
            rsrc_name='foo_step3')
        self.client.resources.signal(stack_identifier, 'foo_step3',
                                     data={'unset_hook': 'pre-create'})
        self.wait_for_event_with_reason(
            stack_identifier,
            reason='Hook pre-create is cleared',
            rsrc_name='foo_step3')
        self._wait_for_stack_status(stack_identifier, 'CREATE_COMPLETE')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.common import test
from heat_tempest_plugin.tests.scenario import scenario_base


@test.requires_resource_type('OS::Octavia::LoadBalancer')
class LoadBalancerTest(scenario_base.ScenarioTestsBase):
    def setUp(self):
        super(LoadBalancerTest, self).setUp()
        self.template_name = 'octavia_lbaas.yaml'
        self.member_template_name = 'lb_member.yaml'
        self.sub_dir = 'templates'

    def _create_stack(self):
        self.parameters = {
            'flavor': self.conf.minimal_instance_type,
            'image': self.conf.minimal_image_ref,
            'network': self.conf.fixed_network_name,
            'subnet': self.conf.fixed_subnet_name
        }
        member_template = self._load_template(
            __file__, self.member_template_name, self.sub_dir
        )
        self.files = {'lb_member.yaml': member_template}
        self.env = {'resource_registry': {
            'OS::Test::PoolMember': self.member_template_name}}

        return self.launch_stack(self.template_name,
                                 parameters=self.parameters,
                                 files=self.files,
                                 environment=self.env)

    @decorators.idempotent_id('5d2c4452-4433-4438-899c-7711c01d3c50')
    def test_create_update_loadbalancer(self):
        statuses = ['PENDING_UPDATE', 'ACTIVE']
        stack_identifier = self._create_stack()
        stack = self.client.stacks.get(stack_identifier)
        output = self._stack_output(stack, 'loadbalancer')
        self.assertIn(output['provisioning_status'], statuses)
        self.parameters['lb_algorithm'] = 'SOURCE_IP'

        self.update_stack(stack_identifier,
                          parameters=self.parameters,
                          existing=True)
        stack = self.client.stacks.get(stack_identifier)

        output = self._stack_output(stack, 'loadbalancer')
        self.assertIn(output['provisioning_status'], statuses)
        output = self._stack_output(stack, 'pool')
        self.assertEqual('SOURCE_IP', output['lb_algorithm'])

    @decorators.idempotent_id('970e91af-1be8-4990-837b-66f9b5aff2b9')
    def test_add_delete_poolmember(self):
        statuses = ['PENDING_UPDATE', 'ACTIVE']
        stack_identifier = self._create_stack()
        stack = self.client.stacks.get(stack_identifier)
        output = self._stack_output(stack, 'loadbalancer')
        self.assertIn(output['provisioning_status'], statuses)
        output = self._stack_output(stack, 'pool')
        self.assertEqual(1, len(output['members']))
        # add pool member
        self.parameters['member_count'] = 2
        self.update_stack(stack_identifier,
                          parameters=self.parameters,
                          existing=True)
        stack = self.client.stacks.get(stack_identifier)

        output = self._stack_output(stack, 'loadbalancer')
        self.assertIn(output['provisioning_status'], statuses)
        output = self._stack_output(stack, 'pool')
        self.assertEqual(2, len(output['members']))
        # delete pool member
        self.parameters['member_count'] = 1
        self.update_stack(stack_identifier,
                          parameters=self.parameters,
                          existing=True)
        stack = self.client.stacks.get(stack_identifier)

        output = self._stack_output(stack, 'loadbalancer')
        self.assertIn(output['provisioning_status'], statuses)
        output = self._stack_output(stack, 'pool')
        self.assertEqual(1, len(output['members']))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


test_template = '''
heat_template_version: 2015-04-30
description: Test template to create/update subnet with allocation_pools.
resources:
  net:
    type: OS::Neutron::Net
  subnet:
    type: OS::Neutron::Subnet
    properties:
      network: { get_resource: net }
      cidr: 11.11.11.0/24
      gateway_ip: 11.11.11.5
      allocation_pools: [{start: 11.11.11.10, end: 11.11.11.250}]
outputs:
  alloc_pools:
    value: {get_attr: [subnet, allocation_pools]}
  gateway_ip:
    value: {get_attr: [subnet, gateway_ip]}
'''


class UpdateSubnetTest(functional_base.FunctionalTestsBase):

    def get_outputs(self, stack_identifier, output_key):
        stack = self.client.stacks.get(stack_identifier)
        output = self._stack_output(stack, output_key)
        return output

    @decorators.idempotent_id('af43fc6d-58ba-4b5f-bd68-07b29f0a96bc')
    def test_update_allocation_pools(self):
        stack_identifier = self.stack_create(template=test_template)
        alloc_pools = self.get_outputs(stack_identifier, 'alloc_pools')
        self.assertEqual([{'start': '11.11.11.10', 'end': '11.11.11.250'}],
                         alloc_pools)

        # Update allocation_pools with a new range
        templ_other_pool = test_template.replace(
            'allocation_pools: [{start: 11.11.11.10, end: 11.11.11.250}]',
            'allocation_pools: [{start: 11.11.11.10, end: 11.11.11.100}]')
        self.update_stack(stack_identifier, templ_other_pool)
        new_alloc_pools = self.get_outputs(stack_identifier, 'alloc_pools')
        # the new pools should be the new range
        self.assertEqual([{'start': '11.11.11.10', 'end': '11.11.11.100'}],
                         new_alloc_pools)

    @decorators.idempotent_id('2f1d3b04-2183-4e50-8aa3-fa0fbff5c861')
    def test_update_allocation_pools_to_empty(self):
        stack_identifier = self.stack_create(template=test_template)
        alloc_pools = self.get_outputs(stack_identifier, 'alloc_pools')
        self.assertEqual([{'start': '11.11.11.10', 'end': '11.11.11.250'}],
                         alloc_pools)

        # Update allocation_pools with []
        templ_empty_pools = test_template.replace(
            'allocation_pools: [{start: 11.11.11.10, end: 11.11.11.250}]',
            'allocation_pools: []')
        self.update_stack(stack_identifier, templ_empty_pools)
        new_alloc_pools = self.get_outputs(stack_identifier, 'alloc_pools')
        # new_alloc_pools should be []
        self.assertEqual([], new_alloc_pools)

    @decorators.idempotent_id('6ec51150-6f83-4179-9faa-a3aee91ea150')
    def test_update_to_no_allocation_pools(self):
        stack_identifier = self.stack_create(template=test_template)
        alloc_pools = self.get_outputs(stack_identifier, 'alloc_pools')
        self.assertEqual([{'start': '11.11.11.10', 'end': '11.11.11.250'}],
                         alloc_pools)

        # Remove the allocation_pools from template
        templ_no_pools = test_template.replace(
            'allocation_pools: [{start: 11.11.11.10, end: 11.11.11.250}]',
            '')
        self.update_stack(stack_identifier, templ_no_pools)
        last_alloc_pools = self.get_outputs(stack_identifier, 'alloc_pools')
        # last_alloc_pools should be []
        self.assertEqual([], last_alloc_pools)

    @decorators.idempotent_id('ffb3eaf8-044b-4229-99f9-5fd983ccc9d9')
    def test_update_gateway_ip(self):
        stack_identifier = self.stack_create(template=test_template)
        gw_ip = self.get_outputs(stack_identifier, 'gateway_ip')
        self.assertEqual('11.11.11.5', gw_ip)

        # Update gateway_ip
        templ_other_gw_ip = test_template.replace(
            'gateway_ip: 11.11.11.5', 'gateway_ip: 11.11.11.9')
        self.update_stack(stack_identifier, templ_other_gw_ip)
        new_gw_ip = self.get_outputs(stack_identifier, 'gateway_ip')
        # the gateway_ip should be the new one
        self.assertEqual('11.11.11.9', new_gw_ip)

    @decorators.idempotent_id('e7b37a6b-e883-4c0c-b6a3-9ece1843e902')
    def test_update_gateway_ip_to_empty(self):
        stack_identifier = self.stack_create(template=test_template)
        gw_ip = self.get_outputs(stack_identifier, 'gateway_ip')
        self.assertEqual('11.11.11.5', gw_ip)

        # Update gateway_ip to null(resolve to '')
        templ_empty_gw_ip = test_template.replace(
            'gateway_ip: 11.11.11.5', 'gateway_ip: null')
        self.update_stack(stack_identifier, templ_empty_gw_ip)
        new_gw_ip = self.get_outputs(stack_identifier, 'gateway_ip')
        # new gateway_ip should be None
        self.assertIsNone(new_gw_ip)

    @decorators.idempotent_id('373bab03-1f66-4883-b1de-2ecc39efe17e')
    def test_update_to_no_gateway_ip(self):
        stack_identifier = self.stack_create(template=test_template)
        gw_ip = self.get_outputs(stack_identifier, 'gateway_ip')
        self.assertEqual('11.11.11.5', gw_ip)

        # Remove the gateway from template
        templ_no_gw_ip = test_template.replace(
            'gateway_ip: 11.11.11.5', '')
        self.update_stack(stack_identifier, templ_no_gw_ip)
        new_gw_ip = self.get_outputs(stack_identifier, 'gateway_ip')
        # new gateway_ip should be None
        self.assertIsNone(new_gw_ip)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from oslo_log import log as logging
from tempest.lib import decorators

from heat_tempest_plugin.common import test
from heat_tempest_plugin.tests.scenario import scenario_base

LOG = logging.getLogger(__name__)


class AodhAlarmTest(scenario_base.ScenarioTestsBase):
    """Class is responsible for testing of aodh usage."""
    def setUp(self):
        super(AodhAlarmTest, self).setUp()
        self.template = self._load_template(__file__,
                                            'test_aodh_alarm.yaml',
                                            'templates')

    def check_instance_count(self, stack_identifier, expected):
        stack = self.client.stacks.get(stack_identifier)
        actual = self._stack_output(stack, 'asg_size')
        if actual != expected:
            LOG.warning('check_instance_count exp:%d, act:%s' % (expected,
                                                                 actual))
        return actual == expected

    @decorators.idempotent_id('fc0f18a6-f65c-4df1-b9c5-e160dea59849')
    def test_alarm(self):
        """Confirm we can create an alarm and trigger it."""
        # create metric
        metric = self.metric_client.metric.create({
            'name': 'my_metric',
            'archive_policy_name': 'high',
        })

        # create the stack
        parameters = {'metric_id': metric['id']}
        stack_identifier = self.stack_create(template=self.template,
                                             parameters=parameters)

        measures = [{'timestamp': test.isotime(datetime.datetime.utcnow()),
                     'value': 100}, {'timestamp': test.isotime(
                         datetime.datetime.utcnow() + datetime.timedelta(
                             minutes=1)), 'value': 100}]
        # send measures(should cause the alarm to fire)
        self.metric_client.metric.add_measures(metric['id'], measures)

        # confirm we get a scaleup.
        # Note: there is little point waiting more than 60s+time to scale up.
        self.assertTrue(test.call_until_true(
            120, 2, self.check_instance_count, stack_identifier, 2))

        # cleanup metric
        self.metric_client.metric.delete(metric['id'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base

test_encryption_vol_type = {
    'heat_template_version': '2015-04-30',
    'description': 'Test template to create encryption volume type.',
    'resources': {
        'my_volume_type': {
            'type': 'OS::Cinder::VolumeType',
            'properties': {
                'name': 'LUKS'
            }
        },
        'my_encrypted_vol_type': {
            'type': 'OS::Cinder::EncryptedVolumeType',
            'properties': {
                'provider': 'nova.volume.encryptors.luks.LuksEncryptor',
                'control_location': 'front-end',
                'cipher': 'aes-xts-plain64',
                'key_size': 512,
                'volume_type': {'get_resource': 'my_volume_type'}
            }
        }
    }
}


class EncryptionVolTypeTest(functional_base.FunctionalTestsBase):
    def setUp(self):
        super(EncryptionVolTypeTest, self).setUp()
        if not self.conf.admin_username or not self.conf.admin_password:
            self.skipTest('No admin creds found, skipping')
        # cinder security policy usage of volume type is limited
        # to being used by administrators only.
        # Switch to admin
        self.setup_clients_for_admin()

    def check_stack(self, sid):
        vt = 'my_volume_type'
        e_vt = 'my_encrypted_vol_type'

        # check if only two resources are present.
        expected_resources = {vt: 'OS::Cinder::VolumeType',
                              e_vt: 'OS::Cinder::EncryptedVolumeType'}
        self.assertEqual(expected_resources,
                         self.list_resources(sid))

        e_vt_obj = self.client.resources.get(sid, e_vt)
        my_encrypted_vol_type_tmpl_prop = test_encryption_vol_type[
            'resources']['my_encrypted_vol_type']['properties']

        # check if the phy rsrc specs was created in accordance with template.
        phy_rsrc_specs = self.volume_client.volume_encryption_types.get(
            e_vt_obj.physical_resource_id)
        self.assertEqual(my_encrypted_vol_type_tmpl_prop['key_size'],
                         phy_rsrc_specs.key_size)
        self.assertEqual(my_encrypted_vol_type_tmpl_prop['provider'],
                         phy_rsrc_specs.provider)
        self.assertEqual(my_encrypted_vol_type_tmpl_prop['cipher'],
                         phy_rsrc_specs.cipher)
        self.assertEqual(my_encrypted_vol_type_tmpl_prop['control_location'],
                         phy_rsrc_specs.control_location)

    @decorators.idempotent_id('a98de4ce-aa59-478b-a8ce-b5d818c76a68')
    def test_create_update(self):
        stack_identifier = self.stack_create(
            template=test_encryption_vol_type)
        self.check_stack(stack_identifier)

        # Change some properties and trigger update.
        my_encrypted_vol_type_tmpl_prop = test_encryption_vol_type[
            'resources']['my_encrypted_vol_type']['properties']
        my_encrypted_vol_type_tmpl_prop['key_size'] = 256
        my_encrypted_vol_type_tmpl_prop['cipher'] = 'aes-cbc-essiv'
        self.update_stack(stack_identifier, test_encryption_vol_type)
        self.check_stack(stack_identifier)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


test_template_depend = {
    'heat_template_version': '2013-05-23',
    'resources': {
        'test1': {
            'type': 'OS::Heat::TestResource',
            'properties': {
                'value': 'Test1',
            }
        },
        'test2': {
            'type': 'OS::Heat::TestResource',
            'depends_on': ['test1'],
            'properties': {
                'value': 'Test2',
            }
        }
    }
}


class ResourcesList(functional_base.FunctionalTestsBase):

    @decorators.idempotent_id('b65b5c82-68b0-42c9-82a0-c0e74e9ad906')
    def test_filtering_with_depend(self):
        stack_identifier = self.stack_create(template=test_template_depend)
        [test2] = self.client.resources.list(stack_identifier,
                                             filters={'name': 'test2'})

        self.assertEqual('CREATE_COMPLETE', test2.resource_status)

    @decorators.idempotent_id('97a65d53-b449-4a43-8283-42d43b165756')
    def test_required_by(self):
        stack_identifier = self.stack_create(template=test_template_depend)
        [test1] = self.client.resources.list(stack_identifier,
                                             filters={'name': 'test1'})

        self.assertEqual(['test2'], test1.required_by)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
import uuid

from heat_tempest_plugin.tests.scenario import scenario_base
from tempest.lib import decorators


class RemoteDeeplyNestedStackTest(scenario_base.ScenarioTestsBase):
    @decorators.idempotent_id('2ed94cae-da14-4060-a6b3-526e7a8cbbe4')
    def test_remote_nested(self):
        parameters = {
            'name': 'remote-nested',
            'network_name': self.conf.floating_network_name,
        }

        stack_id = self.launch_stack(
            template_name='remote_nested_root.yaml',
            parameters={'region': self.conf.region},
            environment={'parameters': parameters}
        )

        stack = self.client.stacks.get(stack_id)
        router_id = self._stack_output(stack, 'router')
        self.assertIsInstance(router_id, six.string_types)
        uuid.UUID(router_id)

        self._stack_delete(stack_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


class StackTagTest(functional_base.FunctionalTestsBase):

    template = '''
heat_template_version: 2014-10-16
description:
  foo
parameters:
  input:
    type: string
    default: test
resources:
  not-used:
    type: OS::Heat::TestResource
    properties:
      wait_secs: 1
      value: {get_param: input}
'''

    @decorators.idempotent_id('67332e51-b427-42d5-ad8d-fd2ec334f361')
    def test_stack_tag(self):
        # Stack create with stack tags
        tags = 'foo,bar'
        stack_identifier = self.stack_create(
            template=self.template,
            tags=tags
        )

        # Ensure property tag is populated and matches given tags
        stack = self.client.stacks.get(stack_identifier)
        self.assertEqual(['foo', 'bar'], stack.tags)

        # Update tags
        updated_tags = 'tag1,tag2'
        self.update_stack(
            stack_identifier,
            template=self.template,
            tags=updated_tags,
            parameters={'input': 'next'})

        # Ensure property tag is populated and matches updated tags
        updated_stack = self.client.stacks.get(stack_identifier)
        self.assertEqual(['tag1', 'tag2'], updated_stack.tags)

        # Delete tags
        self.update_stack(
            stack_identifier,
            template=self.template,
            parameters={'input': 'none'}
        )

        # Ensure property tag is not populated
        empty_tags_stack = self.client.stacks.get(stack_identifier)
        self.assertFalse(empty_tags_stack.tags)

    @decorators.idempotent_id('5ed79584-0684-4f9c-ae8e-44a8f874ec79')
    def test_hidden_stack(self):
        # Stack create with hidden stack tag
        tags = 'foo,%s' % self.conf.hidden_stack_tag
        self.stack_create(
            template=self.template,
            tags=tags)
        # Ensure stack does not exist when we do a stack list
        for stack in self.client.stacks.list():
            self.assertNotIn('hidden', stack.tags, "Hidden stack can be seen")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


class EncryptedParametersTest(functional_base.FunctionalTestsBase):

    template = '''
heat_template_version: 2014-10-16
parameters:
  image:
    type: string
  flavor:
    type: string
  network:
    type: string
  foo:
    type: string
    description: 'parameter with encryption turned on'
    hidden: true
    default: secret
resources:
  server_with_encrypted_property:
    type: OS::Nova::Server
    properties:
      name: { get_param: foo }
      image: { get_param: image }
      flavor: { get_param: flavor }
      networks: [{network: {get_param: network} }]
outputs:
  encrypted_foo_param:
    description: 'encrypted param'
    value: { get_param: foo }
'''

    @decorators.idempotent_id('d5bbf763-8eb9-4657-ad3f-e97405b6e1f2')
    def test_db_encryption(self):
        # Create a stack with the value of 'foo' to be encrypted
        foo_param = 'my_encrypted_foo'
        parameters = {
            "image": self.conf.minimal_image_ref,
            "flavor": self.conf.minimal_instance_type,
            'network': self.conf.fixed_network_name,
            "foo": foo_param
        }

        stack_identifier = self.stack_create(
            template=self.template,
            parameters=parameters
        )
        stack = self.client.stacks.get(stack_identifier)

        # Verify the output value for 'foo' parameter
        for out in stack.outputs:
            if out['output_key'] == 'encrypted_foo_param':
                self.assertEqual(foo_param, out['output_value'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from cinderclient import client as cinder_client
from gnocchiclient import client as gnocchi_client
from heatclient import client as heat_client
from keystoneauth1.identity.generic import password
from keystoneauth1 import session
from neutronclient.v2_0 import client as neutron_client
from novaclient import client as nova_client
from swiftclient import client as swift_client


class KeystoneWrapperClient(object):
    """Wrapper object for keystone client

    This wraps keystone client, so we can encpasulate certain
    added properties like auth_token, project_id etc.
    """
    def __init__(self, auth_plugin, verify=True):
        self.auth_plugin = auth_plugin
        self.session = session.Session(
            auth=auth_plugin,
            verify=verify)

    @property
    def auth_token(self):
        return self.auth_plugin.get_token(self.session)

    @property
    def auth_ref(self):
        return self.auth_plugin.get_access(self.session)

    @property
    def project_id(self):
        return self.auth_plugin.get_project_id(self.session)

    def get_endpoint_url(self, service_type, region=None,
                         endpoint_type='public'):
        kwargs = {
            'service_type': service_type,
            'region_name': region,
            'interface': endpoint_type}
        return self.auth_ref.service_catalog.url_for(**kwargs)


class ClientManager(object):
    """Provides access to the official python clients for calling various APIs.

    Manager that provides access to the official python clients for
    calling various OpenStack APIs.
    """

    CINDERCLIENT_VERSION = '3'
    HEATCLIENT_VERSION = '1'
    NOVA_API_VERSION = '2.1'
    GNOCCHI_VERSION = '1'

    def __init__(self, conf, admin_credentials=False):
        self.conf = conf
        self.admin_credentials = admin_credentials

        self.auth_version = self.conf.auth_version
        if not self.auth_version:
            self.auth_version = self.conf.auth_url.split('/v')[1]
        self.insecure = self.conf.disable_ssl_certificate_validation
        self.ca_file = self.conf.ca_file

        self.identity_client = self._get_identity_client()
        self.orchestration_client = self._get_orchestration_client()
        self.compute_client = self._get_compute_client()
        self.network_client = self._get_network_client()
        self.volume_client = self._get_volume_client()
        self.object_client = self._get_object_client()
        self.metric_client = self._get_metric_client()

    def _username(self):
        if self.admin_credentials:
            return self.conf.admin_username
        return self.conf.username

    def _password(self):
        if self.admin_credentials:
            return self.conf.admin_password
        return self.conf.password

    def _project_name(self):
        if self.admin_credentials:
            return self.conf.admin_project_name
        return self.conf.project_name

    def _get_orchestration_client(self):
        endpoint = os.environ.get('HEAT_URL')
        if os.environ.get('OS_NO_CLIENT_AUTH') == 'True':
            session = None
        else:
            session = self.identity_client.session

        return heat_client.Client(
            self.HEATCLIENT_VERSION,
            endpoint,
            session=session,
            endpoint_type=self.conf.endpoint_type,
            service_type='orchestration',
            region_name=self.conf.region,
            username=self._username(),
            password=self._password())

    def _get_identity_client(self):
        user_domain_id = self.conf.user_domain_id
        project_domain_id = self.conf.project_domain_id
        user_domain_name = self.conf.user_domain_name
        project_domain_name = self.conf.project_domain_name
        kwargs = {
            'username': self._username(),
            'password': self._password(),
            'project_name': self._project_name(),
            'auth_url': self.conf.auth_url
        }
        # keystone v2 can't ignore domain details
        if self.auth_version == '3':
            kwargs.update({
                'user_domain_id': user_domain_id,
                'project_domain_id': project_domain_id,
                'user_domain_name': user_domain_name,
                'project_domain_name': project_domain_name})
        auth = password.Password(**kwargs)
        if self.insecure:
            verify_cert = False
        else:
            verify_cert = self.ca_file or True

        return KeystoneWrapperClient(auth, verify_cert)

    def _get_compute_client(self):
        # Create our default Nova client to use in testing
        return nova_client.Client(
            self.NOVA_API_VERSION,
            session=self.identity_client.session,
            service_type='compute',
            endpoint_type=self.conf.endpoint_type,
            region_name=self.conf.region,
            os_cache=False,
            http_log_debug=True)

    def _get_network_client(self):

        return neutron_client.Client(
            session=self.identity_client.session,
            service_type='network',
            region_name=self.conf.region,
            endpoint_type=self.conf.endpoint_type)

    def _get_volume_client(self):
        return cinder_client.Client(
            self.CINDERCLIENT_VERSION,
            session=self.identity_client.session,
            endpoint_type=self.conf.endpoint_type,
            region_name=self.conf.region,
            http_log_debug=True)

    def _get_object_client(self):
        args = {
            'auth_version': self.auth_version,
            'session': self.identity_client.session,
            'cacert': self.ca_file,
            'insecure': self.insecure,
            'os_options': {'endpoint_type': self.conf.endpoint_type,
                           'region_name': self.conf.region,
                           'service_type': 'object-store'},
        }
        return swift_client.Connection(**args)

    def _get_metric_client(self):

        adapter_options = {'interface': self.conf.endpoint_type,
                           'region_name': self.conf.region}
        args = {
            'session': self.identity_client.session,
            'adapter_options': adapter_options
        }
        return gnocchi_client.Client(version=self.GNOCCHI_VERSION,
                                     **args)
